@@ -44,16 +44,20 @@ import java.util.List;
 import java.util.Map;
 import java.util.concurrent.ConcurrentHashMap;
 
+/**
+ * StringSearchModelInterpolator
+ */
 @Component( role = ModelInterpolator.class )
 public class StringSearchModelInterpolator
     extends AbstractStringBasedModelInterpolator
 {
 
     private static final Map<Class<?>, InterpolateObjectAction.CacheItem> CACHED_ENTRIES =
-        new ConcurrentHashMap<Class<?>, InterpolateObjectAction.CacheItem>( 80, 0.75f, 2 );
+        new ConcurrentHashMap<>( 80, 0.75f, 2 );
     // Empirical data from 3.x, actual =40
 
 
+    @Override
     public Model interpolateModel( Model model, File projectDir, ModelBuildingRequest config,
                                    ModelProblemCollector problems )
     {
@@ -82,6 +86,7 @@ public class StringSearchModelInterpolator
         }
     }
 
+    @Override
     protected Interpolator createInterpolator()
     {
         StringSearchInterpolator interpolator = new StringSearchInterpolator();
@@ -104,15 +109,14 @@ public class StringSearchModelInterpolator
 
         private final ModelProblemCollector problems;
 
-        public InterpolateObjectAction( Object target, List<? extends ValueSource> valueSources,
-                                        List<? extends InterpolationPostProcessor> postProcessors,
-                                        StringSearchModelInterpolator modelInterpolator,
-                                        ModelProblemCollector problems )
+        InterpolateObjectAction( Object target, List<? extends ValueSource> valueSources,
+                                 List<? extends InterpolationPostProcessor> postProcessors,
+                                 StringSearchModelInterpolator modelInterpolator, ModelProblemCollector problems )
         {
             this.valueSources = valueSources;
             this.postProcessors = postProcessors;
 
-            this.interpolationTargets = new LinkedList<Object>();
+            this.interpolationTargets = new LinkedList<>();
             interpolationTargets.add( target );
 
             this.modelInterpolator = modelInterpolator;
@@ -120,6 +124,7 @@ public class StringSearchModelInterpolator
             this.problems = problems;
         }
 
+        @Override
         public Object run()
         {
             while ( !interpolationTargets.isEmpty() )
@@ -228,7 +233,7 @@ public class StringSearchModelInterpolator
             {
                 this.isQualifiedForInterpolation = isQualifiedForInterpolation( clazz );
                 this.isArray = clazz.isArray();
-                List<CacheField> fields = new ArrayList<CacheField>();
+                List<CacheField> fields = new ArrayList<>();
                 for ( Field currentField : clazz.getDeclaredFields() )
                 {
                     Class<?> type = currentField.getType();
@@ -303,7 +308,7 @@ public class StringSearchModelInterpolator
                             new ModelProblemCollectorRequest( Severity.ERROR, Version.BASE ).setMessage(
                                 "Failed to interpolate field3: " + field + " on class: "
                                     + field.getType().getName() ).setException(
-                                e ) ); // todo: Not entirely the same message
+                                e ) ); // TODO Not entirely the same message
                     }
                     catch ( IllegalAccessException e )
                     {
@@ -450,6 +455,7 @@ public class StringSearchModelInterpolator
                             }
                             catch ( UnsupportedOperationException ignore )
                             {
+                                // nop
                             }
                         }
                     }
