@@ -114,7 +114,7 @@ public class CVSPass extends Task {
             String pwdfile = buf.toString() + cvsRoot + " A"
                 + mangle(password);
 
-            log("Writing -> " + pwdfile , Project.MSG_DEBUG);
+            log("Writing -> " + pwdfile, Project.MSG_DEBUG);
 
             writer = new BufferedWriter(new FileWriter(passFile));
 
@@ -129,9 +129,9 @@ public class CVSPass extends Task {
     }
 
     private final String mangle(String password) {
-        StringBuffer buf = new StringBuffer();
-        for (int i = 0; i < password.length(); i++) {
-            buf.append(shifts[password.charAt(i)]);
+        StringBuilder buf = new StringBuilder();
+        for (final char ch : password.toCharArray()) {
+            buf.append(shifts[ch]);
         }
         return buf.toString();
     }
