@@ -150,7 +150,7 @@ public class Jar extends Zip {
     // CheckStyle:LineLength OFF - Link is too long.
     /**
      * Strict mode for checking rules of the JAR-Specification.
-     * @see <a href="https://docs.oracle.com/javase/8/docs/technotes/guides/versioning/spec/versioning2.html#wp89936"></a>
+     * @see http://java.sun.com/j2se/1.3/docs/guide/versioning/spec/VersioningSpecification.html#PackageVersioning
      */
     private StrictMode strict = new StrictMode("ignore");
 
@@ -413,7 +413,7 @@ public class Jar extends Zip {
      */
     public void setFilesetmanifest(FilesetManifestConfig config) {
         filesetManifestConfig = config;
-        mergeManifestsMain = config != null && "merge".equals(config.getValue());
+        mergeManifestsMain = "merge".equals(config.getValue());
 
         if (filesetManifestConfig != null
             && !filesetManifestConfig.getValue().equals("skip")) {
@@ -481,7 +481,6 @@ public class Jar extends Zip {
     /**
      * Whether to merge Class-Path attributes.
      *
-     * @param b boolean
      * @since Ant 1.8.0
      */
     public void setMergeClassPathAttributes(boolean b) {
@@ -492,7 +491,6 @@ public class Jar extends Zip {
      * Whether to flatten multi-valued attributes (i.e. Class-Path)
      * into a single one.
      *
-     * @param b boolean
      * @since Ant 1.8.0
      */
     public void setFlattenAttributes(boolean b) {
@@ -574,8 +572,10 @@ public class Jar extends Zip {
 
     private void writeManifest(ZipOutputStream zOut, Manifest manifest)
         throws IOException {
-        for (String warning : Collections.list(manifest.getWarnings())) {
-            log("Manifest warning: " + warning, Project.MSG_WARN);
+        for (Enumeration<String> e = manifest.getWarnings();
+             e.hasMoreElements();) {
+            log("Manifest warning: " + e.nextElement(),
+                Project.MSG_WARN);
         }
 
         zipDir((Resource) null, zOut, "META-INF/", ZipFileSet.DEFAULT_DIR_MODE,
@@ -659,12 +659,13 @@ public class Jar extends Zip {
                     cpEntries[c++] = tok.nextToken();
                 }
             }
-            for (String indexJarEntry : indexJars.list()) {
-                String name = findJarName(indexJarEntry, cpEntries);
+            String[] indexJarEntries = indexJars.list();
+            for (int i = 0; i < indexJarEntries.length; i++) {
+                String name = findJarName(indexJarEntries[i], cpEntries);
                 if (name != null) {
                     ArrayList<String> dirs = new ArrayList<String>();
                     ArrayList<String> files = new ArrayList<String>();
-                    grabFilesAndDirs(indexJarEntry, dirs, files);
+                    grabFilesAndDirs(indexJarEntries[i], dirs, files);
                     if (dirs.size() + files.size() > 0) {
                         writer.println(name);
                         writeIndexLikeList(dirs, files, writer);
@@ -714,7 +715,7 @@ public class Jar extends Zip {
                            + " be replaced by a newly generated one.",
                            Project.MSG_WARN);
         } else {
-            if (index && !vPath.contains("/")) {
+            if (index && vPath.indexOf("/") == -1) {
                 rootEntries.addElement(vPath);
             }
             super.zipFile(is, zOut, vPath, lastModified, fromArchive, mode);
@@ -827,8 +828,8 @@ public class Jar extends Zip {
             // checks here deferring them for the second run
             Resource[][] manifests = grabManifests(rcs);
             int count = 0;
-            for (Resource[] mf : manifests) {
-                count += mf.length;
+            for (int i = 0; i < manifests.length; i++) {
+                count += manifests[i].length;
             }
             log("found a total of " + count + " manifests in "
                 + manifests.length + " resource collections",
@@ -1031,10 +1032,12 @@ public class Jar extends Zip {
      * @param dirs a list of directories
      * @param files a list of files
      * @param writer the writer to write to
+     * @throws IOException on error
      * @since Ant 1.6.2
      */
     protected final void writeIndexLikeList(List<String> dirs, List<String> files,
-                                            PrintWriter writer) {
+                                            PrintWriter writer)
+        throws IOException {
         // JarIndex is sorting the directories by ascending order.
         // it has no value but cosmetic since it will be read into a
         // hashtable by the classloader, but we'll do so anyway.
@@ -1177,7 +1180,9 @@ public class Jar extends Zip {
             if (rcs[i] instanceof FileSet) {
                 resources = grabResources(new FileSet[] {(FileSet) rcs[i]});
             } else {
-                resources = grabNonFileSetResources(new ResourceCollection[] {rcs[i]});
+                resources = grabNonFileSetResources(new ResourceCollection[] {
+                        rcs[i]
+                    });
             }
             for (int j = 0; j < resources[0].length; j++) {
                 String name = resources[0][j].getName().replace('\\', '/');
@@ -1222,7 +1227,7 @@ public class Jar extends Zip {
          * @return the list of values.
          */
         public String[] getValues() {
-            return new String[] {"fail", "warn", "ignore"};
+            return new String[]{"fail", "warn", "ignore"};
         }
         /**
          * @return The log level according to the strict mode.
