@@ -17,14 +17,14 @@
  */
 package org.apache.tools.ant.taskdefs;
 
-import java.io.ByteArrayInputStream;
-import java.io.ByteArrayOutputStream;
+import java.io.BufferedInputStream;
 import java.io.File;
 import java.io.FileInputStream;
 import java.io.FileOutputStream;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.OutputStream;
+import java.text.ParseException;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collections;
@@ -55,6 +55,7 @@ import org.apache.tools.ant.types.resources.FileResource;
 import org.apache.tools.ant.types.resources.Union;
 import org.apache.tools.ant.types.resources.ZipResource;
 import org.apache.tools.ant.types.resources.selectors.ResourceSelector;
+import org.apache.tools.ant.util.DateUtils;
 import org.apache.tools.ant.util.FileNameMapper;
 import org.apache.tools.ant.util.FileUtils;
 import org.apache.tools.ant.util.GlobPatternMapper;
@@ -78,7 +79,11 @@ import org.apache.tools.zip.ZipOutputStream.UnicodeExtraFieldPolicy;
  */
 public class Zip extends MatchingTask {
     private static final int BUFFER_SIZE = 8 * 1024;
-    private static final int ROUNDUP_MILLIS = 1999; // 2 seconds - 1
+    /**
+     * The granularity of timestamps inside a ZIP archive.
+     */
+    private static final int ZIP_FILE_TIMESTAMP_GRANULARITY = 2000;
+    private static final int ROUNDUP_MILLIS = ZIP_FILE_TIMESTAMP_GRANULARITY - 1;
     // CheckStyle:VisibilityModifier OFF - bc
 
     protected File zipFile;
@@ -97,7 +102,7 @@ public class Zip extends MatchingTask {
     protected String archiveType = "zip";
 
     // For directories:
-    private static final long EMPTY_CRC = new CRC32 ().getValue ();
+    private static final long EMPTY_CRC = new CRC32().getValue();
     protected String emptyBehavior = "skip";
     private final Vector<ResourceCollection> resources = new Vector<ResourceCollection>();
     protected Hashtable<String, String> addedDirs = new Hashtable<String, String>();
@@ -118,6 +123,9 @@ public class Zip extends MatchingTask {
                 }
             };
 
+    private String fixedModTime = null; // User-provided.
+    private long modTimeMillis = 0; // Calculated.
+
     /**
      * If this flag is true, execute() will run most operations twice,
      * the first time with {@link #skipWriting skipWriting} set to
@@ -496,6 +504,7 @@ public class Zip extends MatchingTask {
 
     /**
      * Assume 0 Unix mode is intentional.
+     * @param b boolean
      * @since Ant 1.8.0
      */
     public void setPreserve0Permissions(final boolean b) {
@@ -504,6 +513,7 @@ public class Zip extends MatchingTask {
 
     /**
      * Assume 0 Unix mode is intentional.
+     * @return boolean
      * @since Ant 1.8.0
      */
     public boolean getPreserve0Permissions() {
@@ -512,6 +522,7 @@ public class Zip extends MatchingTask {
 
     /**
      * Whether to set the language encoding flag.
+     * @param b boolean
      * @since Ant 1.8.0
      */
     public void setUseLanguageEncodingFlag(final boolean b) {
@@ -520,6 +531,7 @@ public class Zip extends MatchingTask {
 
     /**
      * Whether the language encoding flag will be used.
+     * @return boolean
      * @since Ant 1.8.0
      */
     public boolean getUseLanguageEnodingFlag() {
@@ -528,6 +540,7 @@ public class Zip extends MatchingTask {
 
     /**
      * Whether Unicode extra fields will be created.
+     * @param b boolean
      * @since Ant 1.8.0
      */
     public void setCreateUnicodeExtraFields(final UnicodeExtraField b) {
@@ -536,6 +549,7 @@ public class Zip extends MatchingTask {
 
     /**
      * Whether Unicode extra fields will be created.
+     * @return boolean
      * @since Ant 1.8.0
      */
     public UnicodeExtraField getCreateUnicodeExtraFields() {
@@ -548,6 +562,7 @@ public class Zip extends MatchingTask {
      *
      * <p>Defaults to false.</p>
      *
+     * @param b boolean
      * @since Ant 1.8.0
      */
     public void setFallBackToUTF8(final boolean b) {
@@ -558,6 +573,7 @@ public class Zip extends MatchingTask {
      * Whether to fall back to UTF-8 if a name cannot be encoded using
      * the specified encoding.
      *
+     * @return boolean
      * @since Ant 1.8.0
      */
     public boolean getFallBackToUTF8() {
@@ -566,6 +582,7 @@ public class Zip extends MatchingTask {
 
     /**
      * Whether Zip64 extensions should be used.
+     * @param b boolean
      * @since Ant 1.9.1
      */
     public void setZip64Mode(final Zip64ModeAttribute b) {
@@ -574,12 +591,37 @@ public class Zip extends MatchingTask {
 
     /**
      * Whether Zip64 extensions will be used.
+     * @return boolean
      * @since Ant 1.9.1
      */
     public Zip64ModeAttribute getZip64Mode() {
         return zip64Mode;
     }
 
+    /**
+     * Set all stored file modification times to {@code time}.
+     * @param time Milliseconds since 1970-01-01 00:00, or
+     *        <code>YYYY-MM-DD{T/ }HH:MM[:SS[.SSS]][ ][&plusmn;ZZ[[:]ZZ]]</code>, or
+     *        <code>MM/DD/YYYY HH:MM[:SS] {AM/PM}</code>, where {a/b} indicates
+     *        that you must choose one of a or b, and [c] indicates that you
+     *        may use or omit c. &plusmn;ZZZZ is the timezone offset, and may be
+     *        literally "Z" to mean GMT.
+     * @since Ant 1.9.10
+     */
+    public void setModificationtime(String time) {
+        fixedModTime = time;
+    }
+
+    /**
+     * The file modification time previously provided to
+     * {@link #setModificationtime(String)} or {@code null} if unset.
+     * @return String
+     * @since Ant 1.9.10
+     */
+    public String getModificationtime() {
+        return fixedModTime;
+    }
+
     /**
      * validate and build
      * @throws BuildException on error
@@ -740,7 +782,7 @@ public class Zip extends MatchingTask {
                 // temporary file
                 if (doUpdate) {
                     if (!renamedFile.delete()) {
-                        log ("Warning: unable to delete temporary file "
+                        log("Warning: unable to delete temporary file "
                             + renamedFile.getName(), Project.MSG_WARN);
                     }
                 }
@@ -777,7 +819,7 @@ public class Zip extends MatchingTask {
     /** rename the zip file. */
     private File renameFile() {
         final File renamedFile = FILE_UTILS.createTempFile(
-            "zip", ".tmp", zipFile.getParentFile(), true, false);
+            getProject(), "zip", ".tmp", zipFile.getParentFile(), true, false);
         try {
             FILE_UTILS.rename(zipFile, renamedFile);
         } catch (final SecurityException e) {
@@ -832,6 +874,17 @@ public class Zip extends MatchingTask {
                                      + archiveType + " file to create!");
         }
 
+        if (fixedModTime != null) {
+            try {
+                modTimeMillis = DateUtils.parseLenientDateTime(fixedModTime).getTime();
+            } catch (ParseException pe) {
+                throw new BuildException("Failed to parse date string " + fixedModTime + ".");
+            }
+            if (roundUp) {
+                modTimeMillis += ROUNDUP_MILLIS;
+            }
+        }
+
         if (zipFile.exists() && !zipFile.isFile()) {
             throw new BuildException(zipFile + " is not a file.");
         }
@@ -1022,8 +1075,7 @@ public class Zip extends MatchingTask {
      * Determine a Resource's Unix mode or return the given default
      * value if not available.
      */
-    private int getUnixMode(final Resource r, final ZipFile zf, final int defaultMode)
-        throws IOException {
+    private int getUnixMode(final Resource r, final ZipFile zf, final int defaultMode) {
 
         int unixMode = defaultMode;
         if (zf != null) {
@@ -1057,7 +1109,7 @@ public class Zip extends MatchingTask {
                 }
                 InputStream is = null;
                 try {
-                    is = zf.getInputStream(ze);
+                    is = new BufferedInputStream(zf.getInputStream(ze));
                     zipFile(is, zOut, prefix + name, ze.getTime(),
                             fromArchive, mode, ze.getExtraFields(true));
                 } finally {
@@ -1068,7 +1120,7 @@ public class Zip extends MatchingTask {
         } else {
             InputStream is = null;
             try {
-                is = r.getInputStream();
+                is = new BufferedInputStream(r.getInputStream());
                 zipFile(is, zOut, prefix + name, r.getLastModified(),
                         fromArchive, mode, r instanceof ZipResource
                         ? ((ZipResource) r).getExtraFields() : null);
@@ -1548,7 +1600,8 @@ public class Zip extends MatchingTask {
         final Resource[] rs = selectFileResources(initial);
         Resource[] result =
             ResourceUtils.selectOutOfDateSources(this, rs, mapper,
-                                                 getZipScanner());
+                                                 getZipScanner(),
+                                                 ZIP_FILE_TIMESTAMP_GRANULARITY);
         if (!doFilesonly) {
             final Union u = new Union();
             u.addAll(Arrays.asList(selectDirectoryResources(initial)));
@@ -1648,7 +1701,7 @@ public class Zip extends MatchingTask {
 
     /**
      * Add a directory to the zip stream.
-     * @param dir  the directort to add to the archive
+     * @param dir  the directory to add to the archive
      * @param zOut the stream to write to
      * @param vPath the name this entry shall have in the archive
      * @param mode the Unix permissions to set.
@@ -1707,20 +1760,22 @@ public class Zip extends MatchingTask {
         addedDirs.put(vPath, vPath);
 
         if (!skipWriting) {
-            final ZipEntry ze = new ZipEntry (vPath);
+            final ZipEntry ze = new ZipEntry(vPath);
 
             // ZIPs store time with a granularity of 2 seconds, round up
             final int millisToAdd = roundUp ? ROUNDUP_MILLIS : 0;
 
-            if (dir != null && dir.isExists()) {
+            if (fixedModTime != null) {
+                ze.setTime(modTimeMillis);
+            } else if (dir != null && dir.isExists()) {
                 ze.setTime(dir.getLastModified() + millisToAdd);
             } else {
                 ze.setTime(System.currentTimeMillis() + millisToAdd);
             }
-            ze.setSize (0);
-            ze.setMethod (ZipEntry.STORED);
+            ze.setSize(0);
+            ze.setMethod(ZipEntry.STORED);
             // This is faintly ridiculous:
-            ze.setCrc (EMPTY_CRC);
+            ze.setCrc(EMPTY_CRC);
             ze.setUnixMode(mode);
 
             if (extra != null) {
@@ -1741,6 +1796,7 @@ public class Zip extends MatchingTask {
     /**
      * Provides the extra fields for the zip entry currently being
      * added to the archive - if any.
+     * @return ZipExtraField[]
      * @since Ant 1.8.0
      */
     protected final ZipExtraField[] getCurrentExtraFields() {
@@ -1750,6 +1806,7 @@ public class Zip extends MatchingTask {
     /**
      * Sets the extra fields for the zip entry currently being
      * added to the archive - if any.
+     * @param extra ZipExtraField[]
      * @since Ant 1.8.0
      */
     protected final void setCurrentExtraFields(final ZipExtraField[] extra) {
@@ -1771,7 +1828,7 @@ public class Zip extends MatchingTask {
      * @since Ant 1.5.2
      * @throws IOException on error
      */
-    protected void zipFile(InputStream in, final ZipOutputStream zOut, final String vPath,
+    protected void zipFile(final InputStream in, final ZipOutputStream zOut, final String vPath,
                            final long lastModified, final File fromArchive, final int mode)
         throws IOException {
         // fromArchive is used in subclasses overriding this method
@@ -1799,9 +1856,14 @@ public class Zip extends MatchingTask {
 
         if (!skipWriting) {
             final ZipEntry ze = new ZipEntry(vPath);
-            ze.setTime(lastModified);
+            ze.setTime(fixedModTime != null ? modTimeMillis : lastModified);
             ze.setMethod(doCompress ? ZipEntry.DEFLATED : ZipEntry.STORED);
-
+            // if the input stream doesn't support mark/reset ability, we wrap it in a
+            // stream that adds that support.
+            // Note: We do *not* close this newly created wrapping input stream, since
+            // we don't "own" the underlying input stream that's passed to us and closing
+            // that is the responsibility of the caller.
+            final InputStream markableInputStream = in.markSupported() ? in : new BufferedInputStream(in);
             /*
              * ZipOutputStream.putNextEntry expects the ZipEntry to
              * know its size and the CRC sum before you start writing
@@ -1812,31 +1874,15 @@ public class Zip extends MatchingTask {
             if (!zOut.isSeekable() && !doCompress) {
                 long size = 0;
                 final CRC32 cal = new CRC32();
-                if (!in.markSupported()) {
-                    // Store data into a byte[]
-                    final ByteArrayOutputStream bos = new ByteArrayOutputStream();
-
-                    final byte[] buffer = new byte[BUFFER_SIZE];
-                    int count = 0;
-                    do {
-                        size += count;
-                        cal.update(buffer, 0, count);
-                        bos.write(buffer, 0, count);
-                        count = in.read(buffer, 0, buffer.length);
-                    } while (count != -1);
-                    in = new ByteArrayInputStream(bos.toByteArray());
-
-                } else {
-                    in.mark(Integer.MAX_VALUE);
-                    final byte[] buffer = new byte[BUFFER_SIZE];
-                    int count = 0;
-                    do {
-                        size += count;
-                        cal.update(buffer, 0, count);
-                        count = in.read(buffer, 0, buffer.length);
-                    } while (count != -1);
-                    in.reset();
-                }
+                markableInputStream.mark(Integer.MAX_VALUE);
+                final byte[] buffer = new byte[BUFFER_SIZE];
+                int count = 0;
+                do {
+                    size += count;
+                    cal.update(buffer, 0, count);
+                    count = markableInputStream.read(buffer, 0, buffer.length);
+                } while (count != -1);
+                markableInputStream.reset();
                 ze.setSize(size);
                 ze.setCrc(cal.getValue());
             }
@@ -1855,7 +1901,7 @@ public class Zip extends MatchingTask {
                 if (count != 0) {
                     zOut.write(buffer, 0, count);
                 }
-                count = in.read(buffer, 0, buffer.length);
+                count = markableInputStream.read(buffer, 0, buffer.length);
             } while (count != -1);
         }
         addedFiles.addElement(vPath);
@@ -1911,14 +1957,14 @@ public class Zip extends MatchingTask {
                                      getLocation());
         }
 
-        final FileInputStream fIn = new FileInputStream(file);
+        final BufferedInputStream bIn = new BufferedInputStream(new FileInputStream(file));
         try {
             // ZIPs store time with a granularity of 2 seconds, round up
-            zipFile(fIn, zOut, vPath,
+            zipFile(bIn, zOut, vPath,
                     file.lastModified() + (roundUp ? ROUNDUP_MILLIS : 0),
                     null, mode);
         } finally {
-            fIn.close();
+            bIn.close();
         }
     }
 
@@ -2070,6 +2116,7 @@ public class Zip extends MatchingTask {
     /**
      * Drops all resources from the given array that are not selected
      * @param orig the resources to filter
+     * @param selector ResourceSelector
      * @return the filters resources
      * @since Ant 1.8.0
      */
@@ -2096,6 +2143,8 @@ public class Zip extends MatchingTask {
      * Logs a message at the given output level, but only if this is
      * the pass that will actually create the archive.
      *
+     * @param msg String
+     * @param level int
      * @since Ant 1.8.0
      */
     protected void logWhenWriting(final String msg, final int level) {
@@ -2111,8 +2160,8 @@ public class Zip extends MatchingTask {
     public static class Duplicate extends EnumeratedAttribute {
         /**
          * @see EnumeratedAttribute#getValues()
+         * {@inheritDoc}
          */
-        /** {@inheritDoc} */
         @Override
         public String[] getValues() {
             return new String[] {"add", "preserve", "fail"};
@@ -2170,7 +2219,7 @@ public class Zip extends MatchingTask {
     }
 
     /**
-     * Policiy for creation of Unicode extra fields: never, always or
+     * Policy for creation of Unicode extra fields: never, always or
      * not-encodeable.
      *
      * @since Ant 1.8.0
