@@ -70,6 +70,7 @@ public abstract class DefaultCompilerAdapter
     protected boolean depend = false;
     protected boolean verbose = false;
     protected String target;
+    protected String release;
     protected Path bootclasspath;
     protected Path extdirs;
     protected Path compileClasspath;
@@ -111,6 +112,7 @@ public abstract class DefaultCompilerAdapter
         depend = attributes.getDepend();
         verbose = attributes.getVerbose();
         target = attributes.getTarget();
+        release = attributes.getRelease();
         bootclasspath = attributes.getBootclasspath();
         extdirs = attributes.getExtdirs();
         compileList = attributes.getFileList();
@@ -297,7 +299,7 @@ public abstract class DefaultCompilerAdapter
 
         cmd.createArgument().setValue("-classpath");
 
-        // Just add "sourcepath" to classpath ( for JDK1.1 )
+        // Just add "sourcepath" to classpath (for JDK1.1)
         // as well as "bootclasspath" and "extdirs"
         if (assumeJava11()) {
             final Path cp = new Path(project);
@@ -321,15 +323,17 @@ public abstract class DefaultCompilerAdapter
                 cmd.createArgument().setValue("-sourcepath");
                 cmd.createArgument().setPath(sourcepath);
             }
-            if (target != null) {
-                cmd.createArgument().setValue("-target");
-                cmd.createArgument().setValue(target);
-            }
+            if (release == null || !assumeJava9Plus()) {
+                if (target != null) {
+                    cmd.createArgument().setValue("-target");
+                    cmd.createArgument().setValue(target);
+                }
 
-            final Path bp = getBootClassPath();
-            if (bp.size() > 0) {
-                cmd.createArgument().setValue("-bootclasspath");
-                cmd.createArgument().setPath(bp);
+                final Path bp = getBootClassPath();
+                if (bp.size() > 0) {
+                    cmd.createArgument().setValue("-bootclasspath");
+                    cmd.createArgument().setPath(bp);
+                }
             }
 
             if (extdirs != null && extdirs.size() > 0) {
@@ -390,30 +394,54 @@ public abstract class DefaultCompilerAdapter
         setupJavacCommandlineSwitches(cmd, true);
         if (!assumeJava13()) { // -source added with JDK 1.4
             final String t = attributes.getTarget();
-            if (attributes.getSource() != null) {
-                cmd.createArgument().setValue("-source");
-                cmd.createArgument()
-                    .setValue(adjustSourceValue(attributes.getSource()));
+            final String s = attributes.getSource();
+            if (release == null || !assumeJava9Plus()) {
+                if (release != null) {
+                    attributes.log("Support for javac --release has been added"
+                                   + " in Java9 ignoring it");
+                }
+                if (s != null) {
+                    cmd.createArgument().setValue("-source");
+                    cmd.createArgument().setValue(adjustSourceValue(s));
 
-            } else if (t != null && mustSetSourceForTarget(t)) {
-                setImplicitSourceSwitch(cmd, t, adjustSourceValue(t));
+                } else if (t != null && mustSetSourceForTarget(t)) {
+                    setImplicitSourceSwitch(cmd, t, adjustSourceValue(t));
+                }
+            } else { // Java 9+ and release has been set
+                if (t != null || s != null || getBootClassPath().size() > 0) {
+                    attributes.log("Ignoring source, target and bootclasspath"
+                                   + " as release has been set",
+                                   Project.MSG_WARN);
+                }
+                cmd.createArgument().setValue("--release");
+                cmd.createArgument().setValue(release);
             }
         }
         final Path msp = getModulesourcepath();
         if (msp.size() > 0) {
-            cmd.createArgument().setValue("-modulesourcepath");
+            cmd.createArgument().setValue("--module-source-path");
             cmd.createArgument().setPath(msp);
         }
         final Path mp = getModulepath();
         if (mp.size() > 0) {
-            cmd.createArgument().setValue("-modulepath");
+            cmd.createArgument().setValue("--module-path");
             cmd.createArgument().setPath(mp);
         }
         final Path ump = getUpgrademodulepath();
         if (ump.size() > 0) {
-            cmd.createArgument().setValue("-upgrademodulepath");
+            cmd.createArgument().setValue("--upgrade-module-path");
             cmd.createArgument().setPath(ump);
         }
+        if (attributes.getNativeHeaderDir() != null) {
+            if (assumeJava13() || assumeJava14() || assumeJava15() || assumeJava16()
+                    || assumeJava17()) {
+                attributes.log("Support for javac -h has been added in Java8,"
+                               + " ignoring it");
+            } else {
+                cmd.createArgument().setValue("-h");
+                cmd.createArgument().setFile(attributes.getNativeHeaderDir());
+            }
+        }
         return cmd;
     }
 
@@ -457,8 +485,7 @@ public abstract class DefaultCompilerAdapter
      * @param cmd the command line
      */
     protected void logAndAddFilesToCompile(final Commandline cmd) {
-        attributes.log("Compilation " + cmd.describeArguments(),
-                       Project.MSG_VERBOSE);
+        attributes.log("Compilation " + cmd.describeArguments(), Project.MSG_VERBOSE);
 
         final StringBuffer niceSourceList = new StringBuffer("File");
         if (compileList.length != 1) {
@@ -529,10 +556,10 @@ public abstract class DefaultCompilerAdapter
                 BufferedWriter out = null;
                 try {
                     tmpFile = FILE_UTILS.createTempFile(
-                        "files", "", getJavac().getTempdir(), true, true);
+                        getProject(), "files", "", getJavac().getTempdir(), true, true);
                     out = new BufferedWriter(new FileWriter(tmpFile));
                     for (int i = firstFileName; i < args.length; i++) {
-                        if (quoteFiles && args[i].indexOf(" ") > -1) {
+                        if (quoteFiles && args[i].contains(" ")) {
                             args[i] = args[i].replace(File.separatorChar, '/');
                             out.write("\"" + args[i] + "\"");
                         } else {
@@ -672,12 +699,38 @@ public abstract class DefaultCompilerAdapter
     }
 
     /**
-     * Shall we assume JDK 1.9 command line switches?
-     * @return true if JDK 1.9
+     * Shall we assume JDK 9 command line switches?
+     * @return true if JDK 9
      * @since Ant 1.9.4
+     * @deprecated use #assumeJava9 instead
      */
     protected boolean assumeJava19() {
-        return assumeJavaXY("javac1.9", JavaEnvUtils.JAVA_1_9);
+        return assumeJavaXY("javac1.9", JavaEnvUtils.JAVA_9)
+            || assumeJavaXY("javac9", JavaEnvUtils.JAVA_9);
+    }
+
+    /**
+     * Shall we assume JDK 9 command line switches?
+     * @return true if JDK 9
+     * @since Ant 1.9.8
+     */
+    protected boolean assumeJava9() {
+        return assumeJava19();
+    }
+
+    /**
+     * Shall we assume JDK 9+ command line switches?
+     * @return true if JDK 9+
+     * @since Ant 1.9.10
+     */
+    protected boolean assumeJava9Plus() {
+        return "javac1.9".equals(attributes.getCompilerVersion())
+            || "javac9".equals(attributes.getCompilerVersion())
+            || "javac10+".equals(attributes.getCompilerVersion())
+            || (JavaEnvUtils.isAtLeastJavaVersion(JavaEnvUtils.JAVA_9)
+                && ("classic".equals(attributes.getCompilerVersion())
+                || "modern".equals(attributes.getCompilerVersion())
+                || "extJavac".equals(attributes.getCompilerVersion())));
     }
 
     /**
@@ -686,12 +739,10 @@ public abstract class DefaultCompilerAdapter
      */
     private boolean assumeJavaXY(final String javacXY, final String javaEnvVersionXY) {
         return javacXY.equals(attributes.getCompilerVersion())
-            || ("classic".equals(attributes.getCompilerVersion())
-                && JavaEnvUtils.isJavaVersion(javaEnvVersionXY))
-            || ("modern".equals(attributes.getCompilerVersion())
-                && JavaEnvUtils.isJavaVersion(javaEnvVersionXY))
-            || ("extJavac".equals(attributes.getCompilerVersion())
-                && JavaEnvUtils.isJavaVersion(javaEnvVersionXY));
+            || (JavaEnvUtils.isJavaVersion(javaEnvVersionXY)
+                && ("classic".equals(attributes.getCompilerVersion())
+                 || "modern".equals(attributes.getCompilerVersion())
+                 || "extJavac".equals(attributes.getCompilerVersion())));
     }
 
     /**
@@ -755,8 +806,8 @@ public abstract class DefaultCompilerAdapter
         if (assumeJava18()) {
             return "1.8 in JDK 1.8";
         }
-        if (assumeJava19()) {
-            return "1.9 in JDK 1.9";
+        if (assumeJava9Plus()) {
+            return "9 in JDK 9";
         }
         return "";
     }
@@ -778,11 +829,10 @@ public abstract class DefaultCompilerAdapter
             t = t.substring(2);
         }
         return t.equals("1") || t.equals("2") || t.equals("3") || t.equals("4")
-            || ((t.equals("5") || t.equals("6"))
-                && !assumeJava15() && !assumeJava16())
+            || ((t.equals("5") || t.equals("6")) && !assumeJava15() && !assumeJava16())
             || (t.equals("7") && !assumeJava17())
             || (t.equals("8") && !assumeJava18())
-            || (t.equals("9") && !assumeJava19());
+            || (t.equals("9") && !assumeJava9Plus());
     }
 
 
