@@ -144,7 +144,7 @@ public class jlink {
      * a rational manner to outfile.
      * @throws Exception on error.
      */
-    public void link() throws Exception {
+    public void link() throws Exception { //NOSONAR
         ZipOutputStream output = new ZipOutputStream(new FileOutputStream(outfile));
 
         if (compression) {
@@ -223,51 +223,52 @@ public class jlink {
             return;
         }
         ZipFile zipf = new ZipFile(f);
-        Enumeration entries = zipf.entries();
+        try {
+            Enumeration entries = zipf.entries();
 
-        while (entries.hasMoreElements()) {
-            ZipEntry inputEntry = (ZipEntry) entries.nextElement();
-            //Ignore manifest entries.  They're bound to cause conflicts between
-            //files that are being merged.  User should supply their own
-            //manifest file when doing the merge.
-            String inputEntryName = inputEntry.getName();
-            int index = inputEntryName.indexOf("META-INF");
+            while (entries.hasMoreElements()) {
+                ZipEntry inputEntry = (ZipEntry) entries.nextElement();
+                //Ignore manifest entries.  They're bound to cause conflicts between
+                //files that are being merged.  User should supply their own
+                //manifest file when doing the merge.
+                String inputEntryName = inputEntry.getName();
+                int index = inputEntryName.indexOf("META-INF");
 
-            if (index < 0) {
-                //META-INF not found in the name of the entry. Go ahead and process it.
-                try {
-                    output.putNextEntry(processEntry(zipf, inputEntry));
-                } catch (ZipException ex) {
-                    //If we get here, it could be because we are trying to put a
-                    //directory entry that already exists.
-                    //For example, we're trying to write "com", but a previous
-                    //entry from another mergefile was called "com".
-                    //In that case, just ignore the error and go on to the
-                    //next entry.
-                    String mess = ex.getMessage();
-
-                    if (mess.indexOf("duplicate") >= 0) {
-                        //It was the duplicate entry.
-                        continue;
-                    } else {
-                        // I hate to admit it, but we don't know what happened
-                        // here.  Throw the Exception.
-                        throw ex;
+                if (index < 0) {
+                    //META-INF not found in the name of the entry. Go ahead and process it.
+                    try {
+                        output.putNextEntry(processEntry(zipf, inputEntry));
+                    } catch (ZipException ex) {
+                        //If we get here, it could be because we are trying to put a
+                        //directory entry that already exists.
+                        //For example, we're trying to write "com", but a previous
+                        //entry from another mergefile was called "com".
+                        //In that case, just ignore the error and go on to the
+                        //next entry.
+                        if (ex.getMessage().contains("duplicate")) {
+                            //It was the duplicate entry.
+                            continue;
+                        } else {
+                            // I hate to admit it, but we don't know what happened
+                            // here.  Throw the Exception.
+                            throw ex;
+                        }
                     }
-                }
 
-                InputStream in = zipf.getInputStream(inputEntry);
-                int len = buffer.length;
-                int count = -1;
+                    InputStream in = zipf.getInputStream(inputEntry);
+                    int len = buffer.length;
+                    int count = -1;
 
-                while ((count = in.read(buffer, 0, len)) > 0) {
-                    output.write(buffer, 0, count);
+                    while ((count = in.read(buffer, 0, len)) > 0) {
+                        output.write(buffer, 0, count);
+                    }
+                    in.close();
+                    output.closeEntry();
                 }
-                in.close();
-                output.closeEntry();
             }
+        } finally {
+            zipf.close();
         }
-        zipf.close();
     }
 
 
@@ -276,10 +277,7 @@ public class jlink {
      */
     private void addDirContents(ZipOutputStream output, File dir, String prefix,
                                 boolean compress) throws IOException {
-        String[] contents = dir.list();
-
-        for (int i = 0; i < contents.length; ++i) {
-            String name = contents[i];
+        for (String name : dir.list()) {
             File file = new File(dir, name);
 
             if (file.isDirectory()) {
