@@ -19,8 +19,10 @@ package org.apache.tools.ant.types;
 
 import java.io.BufferedReader;
 import java.io.File;
+import java.io.FileInputStream;
 import java.io.FileReader;
 import java.io.IOException;
+import java.io.InputStreamReader;
 import java.util.ArrayList;
 import java.util.List;
 import java.util.StringTokenizer;
@@ -40,8 +42,8 @@ import org.apache.tools.ant.util.FileUtils;
 public class PatternSet extends DataType implements Cloneable {
     private List<NameEntry> includeList = new ArrayList<NameEntry>();
     private List<NameEntry> excludeList = new ArrayList<NameEntry>();
-    private List<NameEntry> includesFileList = new ArrayList<NameEntry>();
-    private List<NameEntry> excludesFileList = new ArrayList<NameEntry>();
+    private List<PatternFileNameEntry> includesFileList = new ArrayList<PatternFileNameEntry>();
+    private List<PatternFileNameEntry> excludesFileList = new ArrayList<PatternFileNameEntry>();
 
     /**
      * inner class to hold a name on list.  "If" and "Unless" attributes
@@ -176,6 +178,47 @@ public class PatternSet extends DataType implements Cloneable {
         }
     }
 
+    /**
+     * Adds encoding support to {@link NameEntry}.
+     * @since Ant 1.9.12
+     */
+    public class PatternFileNameEntry extends NameEntry {
+        private String encoding;
+
+        /**
+         * Encoding to use when reading the file, defaults to the platform's default
+         * encoding.
+         *
+         * <p>
+         * For a list of possible values see
+         * <a href="https://docs.oracle.com/javase/1.5.0/docs/guide/intl/encoding.doc.html">
+         * https://docs.oracle.com/javase/1.5.0/docs/guide/intl/encoding.doc.html</a>.
+         * </p>
+         *
+         * @param encoding String
+         */
+        public final void setEncoding(String encoding) {
+            this.encoding = encoding;
+        }
+
+        /**
+         * Encoding to use when reading the file, defaults to the platform's default
+         * encoding.
+         *
+         * @return the encoding name
+         */
+        public final String getEncoding() {
+            return encoding;
+        }
+
+        @Override
+        public String toString() {
+            String baseString = super.toString();
+            return encoding == null ? baseString
+                : new StringBuilder(baseString).append(";encoding->").append(encoding).toString();
+        }
+    }
+
     private static final class InvertedPatternSet extends PatternSet {
         private InvertedPatternSet(PatternSet p) {
             setProject(p.getProject());
@@ -225,13 +268,13 @@ public class PatternSet extends DataType implements Cloneable {
         String[] nestedExcludes = p.getExcludePatterns(getProject());
 
         if (nestedIncludes != null) {
-            for (int i = 0; i < nestedIncludes.length; i++) {
-                createInclude().setName(nestedIncludes[i]);
+            for (String nestedInclude : nestedIncludes) {
+                createInclude().setName(nestedInclude);
             }
         }
         if (nestedExcludes != null) {
-            for (int i = 0; i < nestedExcludes.length; i++) {
-                createExclude().setName(nestedExcludes[i]);
+            for (String nestedExclude : nestedExcludes) {
+                createExclude().setName(nestedExclude);
             }
         }
     }
@@ -255,7 +298,7 @@ public class PatternSet extends DataType implements Cloneable {
         if (isReference()) {
             throw noChildrenAllowed();
         }
-        return addPatternToList(includesFileList);
+        return addPatternFileToList(includesFileList);
     }
 
     /**
@@ -277,7 +320,7 @@ public class PatternSet extends DataType implements Cloneable {
         if (isReference()) {
             throw noChildrenAllowed();
         }
-        return addPatternToList(excludesFileList);
+        return addPatternFileToList(excludesFileList);
     }
 
     /**
@@ -325,6 +368,15 @@ public class PatternSet extends DataType implements Cloneable {
         return result;
     }
 
+    /**
+     * add a pattern file name entry to the given list
+     */
+    private PatternFileNameEntry addPatternFileToList(List<PatternFileNameEntry> list) {
+        PatternFileNameEntry result = new PatternFileNameEntry();
+        list.add(result);
+        return result;
+    }
+
     /**
      * Sets the name of the file containing the includes patterns.
      *
@@ -355,13 +407,17 @@ public class PatternSet extends DataType implements Cloneable {
      *  Reads path matching patterns from a file and adds them to the
      *  includes or excludes list (as appropriate).
      */
-    private void readPatterns(File patternfile, List<NameEntry> patternlist, Project p)
+    private void readPatterns(File patternfile, String encoding, List<NameEntry> patternlist, Project p)
             throws BuildException {
 
         BufferedReader patternReader = null;
         try {
             // Get a FileReader
-            patternReader = new BufferedReader(new FileReader(patternfile));
+            if (encoding == null) {
+                patternReader = new BufferedReader(new FileReader(patternfile));
+            } else {
+                patternReader = new BufferedReader(new InputStreamReader(new FileInputStream(patternfile), encoding));
+            }
 
             // Create one NameEntry in the appropriate pattern list for each
             // line in the file.
@@ -393,14 +449,14 @@ public class PatternSet extends DataType implements Cloneable {
         dieOnCircularReference(p);
         String[] incl = other.getIncludePatterns(p);
         if (incl != null) {
-            for (int i = 0; i < incl.length; i++) {
-                createInclude().setName(incl[i]);
+            for (String include : incl) {
+                createInclude().setName(include);
             }
         }
         String[] excl = other.getExcludePatterns(p);
         if (excl != null) {
-            for (int i = 0; i < excl.length; i++) {
-                createExclude().setName(excl[i]);
+            for (String exclude : excl) {
+                createExclude().setName(exclude);
             }
         }
     }
@@ -478,7 +534,7 @@ public class PatternSet extends DataType implements Cloneable {
      */
     private void readFiles(Project p) {
         if (includesFileList.size() > 0) {
-            for (NameEntry ne : includesFileList) {
+            for (PatternFileNameEntry ne : includesFileList) {
                 String fileName = ne.evalName(p);
                 if (fileName != null) {
                     File inclFile = p.resolveFile(fileName);
@@ -486,13 +542,13 @@ public class PatternSet extends DataType implements Cloneable {
                         throw new BuildException("Includesfile " + inclFile.getAbsolutePath()
                                 + " not found.");
                     }
-                    readPatterns(inclFile, includeList, p);
+                    readPatterns(inclFile, ne.getEncoding(), includeList, p);
                 }
             }
             includesFileList.clear();
         }
         if (excludesFileList.size() > 0) {
-            for (NameEntry ne : excludesFileList) {
+            for (PatternFileNameEntry ne : excludesFileList) {
                 String fileName = ne.evalName(p);
                 if (fileName != null) {
                     File exclFile = p.resolveFile(fileName);
@@ -500,7 +556,7 @@ public class PatternSet extends DataType implements Cloneable {
                         throw new BuildException("Excludesfile " + exclFile.getAbsolutePath()
                                 + " not found.");
                     }
-                    readPatterns(exclFile, excludeList, p);
+                    readPatterns(exclFile, ne.getEncoding(), excludeList, p);
                 }
             }
             excludesFileList.clear();
@@ -523,8 +579,8 @@ public class PatternSet extends DataType implements Cloneable {
             PatternSet ps = (PatternSet) super.clone();
             ps.includeList = new ArrayList<NameEntry>(includeList);
             ps.excludeList = new ArrayList<NameEntry>(excludeList);
-            ps.includesFileList = new ArrayList<NameEntry>(includesFileList);
-            ps.excludesFileList = new ArrayList<NameEntry>(excludesFileList);
+            ps.includesFileList = new ArrayList<PatternFileNameEntry>(includesFileList);
+            ps.excludesFileList = new ArrayList<PatternFileNameEntry>(excludesFileList);
             return ps;
         } catch (CloneNotSupportedException e) {
             throw new BuildException(e);
