@@ -22,9 +22,11 @@ import org.codehaus.groovy.GroovyBugError;
 import org.codehaus.groovy.ast.GroovyCodeVisitor;
 import org.codehaus.groovy.syntax.Token;
 
+import static org.apache.groovy.ast.tools.ClassNodeUtils.formatTypeName;
+
 /**
  * Represents one or more local variables. Typically it is a single local variable
- * declared by name with an expression like "def foo" or with type "String foo". However, 
+ * declared by name with an expression like "def foo" or with type "String foo". However,
  * the multiple assignment feature allows you to create two or more variables using
  * an expression like: <code>def (x, y) = [1, 2]</code>.
  * <p>
@@ -42,35 +44,35 @@ import org.codehaus.groovy.syntax.Token;
  * is not appropriate is unsafe and will result in a <code>ClassCastException</code>.
  */
 public class DeclarationExpression extends BinaryExpression {
-    
+
     /**
-     * Creates a DeclarationExpression for VariableExpressions like "def x" or "String y = 'foo'". 
+     * Creates a DeclarationExpression for VariableExpressions like "def x" or "String y = 'foo'".
      * @param left
      *      the left hand side of a variable declaration
      * @param operation
      *      the operation, typically an assignment operator
      * @param right
      *      the right hand side of a declaration
-     */ 
+     */
     public DeclarationExpression(VariableExpression left, Token operation, Expression right) {
         super(left,operation,right);
     }
-    
+
     /**
      * Creates a DeclarationExpression for Expressions like "def (x, y) = [1, 2]"
      * @param left
-     *      the left hand side of a declaration. Must be either a VariableExpression or 
-     *      a TupleExpression with at least one element.  
+     *      the left hand side of a declaration. Must be either a VariableExpression or
+     *      a TupleExpression with at least one element.
      * @param operation
      *       the operation, typically an assignment operator
      * @param right
      *       the right hand side of a declaration
-     */ 
+     */
     public DeclarationExpression(Expression left, Token operation, Expression right) {
         super(left,operation,right);
         check(left);
     }
-    
+
     private static void check(Expression left) {
         if (left instanceof VariableExpression) {
             //nothing
@@ -82,6 +84,7 @@ public class DeclarationExpression extends BinaryExpression {
         }
     }
 
+    @Override
     public void visit(GroovyCodeVisitor visitor) {
         visitor.visitDeclarationExpression(this);
     }
@@ -90,7 +93,7 @@ public class DeclarationExpression extends BinaryExpression {
      * This method returns the left hand side of the declaration cast to the VariableExpression type.
      * This is an unsafe method to call. In a multiple assignment statement, the left hand side will
      * be a TupleExpression and a ClassCastException will occur. If you invoke this method then
-     * be sure to invoke isMultipleAssignmentDeclaration() first to check that it is safe to do so. 
+     * be sure to invoke isMultipleAssignmentDeclaration() first to check that it is safe to do so.
      * If that method returns true then this method is safe to call.
      *
      * @return left hand side of normal variable declarations
@@ -99,38 +102,74 @@ public class DeclarationExpression extends BinaryExpression {
     public VariableExpression getVariableExpression() {
         return (VariableExpression) this.getLeftExpression();
     }
-    
+
     /**
      * This method returns the left hand side of the declaration cast to the TupleExpression type.
      * This is an unsafe method to call. In a single assignment statement, the left hand side will
      * be a VariableExpression and a ClassCastException will occur. If you invoke this method then
-     * be sure to invoke isMultipleAssignmentDeclaration() first to check that it is safe to do so. 
-     * If that method returns true then this method is safe to call. 
+     * be sure to invoke isMultipleAssignmentDeclaration() first to check that it is safe to do so.
+     * If that method returns true then this method is safe to call.
      * @return
      *      left hand side of multiple assignment declarations
-     * @throws ClassCastException 
+     * @throws ClassCastException
      *      if the left hand side is not a TupleExpression (and is probably a VariableExpression).
      *
      */
     public TupleExpression getTupleExpression() {
         return (TupleExpression) this.getLeftExpression();
     }
-    
+
+    @Override
+    public String getText() {
+        StringBuilder text = new StringBuilder();
+
+        if (!isMultipleAssignmentDeclaration()) {
+            VariableExpression v = getVariableExpression();
+            if (v.isDynamicTyped()) {
+                text.append("def");
+            } else {
+                text.append(formatTypeName(v.getType()));
+            }
+            text.append(' ').append(v.getText());
+        } else {
+            TupleExpression t = getTupleExpression();
+            text.append("def (");
+            for (Expression e : t.getExpressions()) {
+                if (e instanceof VariableExpression) {
+                    VariableExpression v = (VariableExpression) e;
+                    if (!v.isDynamicTyped()) {
+                        text.append(formatTypeName(v.getType())).append(' ');
+                    }
+                }
+                text.append(e.getText()).append(", ");
+            }
+            text.setLength(text.length() - 2);
+            text.append(')');
+        }
+        text.append(' ').append(getOperation().getText());
+        text.append(' ').append(getRightExpression().getText());
+
+        return text.toString();
+    }
+
     /**
      * This method sets the leftExpression for this BinaryExpression. The parameter must be
-     * either a VariableExpression or a TupleExpression with one or more elements. 
+     * either a VariableExpression or a TupleExpression with one or more elements.
      * @param leftExpression
-     *      either a VariableExpression or a TupleExpression with one or more elements. 
-     */ 
+     *      either a VariableExpression or a TupleExpression with one or more elements.
+     */
+    @Override
     public void setLeftExpression(Expression leftExpression) {
         check(leftExpression);
         super.setLeftExpression(leftExpression);
     }
-    
+
+    @Override
     public void setRightExpression(Expression rightExpression) {
         super.setRightExpression(rightExpression);
     }
-    
+
+    @Override
     public Expression transformExpression(ExpressionTransformer transformer) {
         Expression ret = new DeclarationExpression(transformer.transform(getLeftExpression()),
                 getOperation(), transformer.transform(getRightExpression()));
@@ -140,18 +179,17 @@ public class DeclarationExpression extends BinaryExpression {
         ret.copyNodeMetaData(this);
         return ret;
     }
-    
+
     /**
-     * This method tells you if this declaration is a multiple assignment declaration, which 
+     * This method tells you if this declaration is a multiple assignment declaration, which
      * has the form "def (x, y) = ..." in Groovy. If this method returns true, then the left
-     * hand side is an ArgumentListExpression. Do not call "getVariableExpression()" on this 
-     * object if this method returns true, instead use "getLeftExpression()". 
+     * hand side is an ArgumentListExpression. Do not call "getVariableExpression()" on this
+     * object if this method returns true, instead use "getLeftExpression()".
      * @return
      *      true if this declaration is a multiple assignment declaration, which means the
-     *      left hand side is an ArgumentListExpression. 
-     */ 
+     *      left hand side is an ArgumentListExpression.
+     */
     public boolean isMultipleAssignmentDeclaration() {
         return getLeftExpression() instanceof TupleExpression;
     }
 }
-
