@@ -58,8 +58,6 @@ import static org.objectweb.asm.Opcodes.H_INVOKESTATIC;
 /**
  * This Writer is used to generate the call invocation byte codes
  * for usage by invokedynamic.
- * 
- * @author <a href="mailto:blackdrag@gmx.org">Jochen "blackdrag" Theodorou</a>
  */
 public class InvokeDynamicWriter extends InvocationWriter {
     
@@ -77,7 +75,7 @@ public class InvokeDynamicWriter extends InvocationWriter {
                 "bootstrap",
                 BSM_METHOD_TYPE_DESCRIPTOR);
 
-    private WriterController controller;
+    private final WriterController controller;
 
     public InvokeDynamicWriter(WriterController wc) {
         super(wc);
@@ -127,7 +125,7 @@ public class InvokeDynamicWriter extends InvocationWriter {
     private void makeIndyCall(MethodCallerMultiAdapter adapter, Expression receiver, boolean implicitThis, boolean safe, String methodName, Expression arguments) {
         OperandStack operandStack = controller.getOperandStack();
         
-        String sig = prepareIndyCall(receiver, implicitThis);
+        StringBuilder sig = new StringBuilder(prepareIndyCall(receiver, implicitThis));
         
         // load arguments
         int numberOfArguments = 1;
@@ -135,26 +133,26 @@ public class InvokeDynamicWriter extends InvocationWriter {
         boolean containsSpreadExpression = AsmClassGenerator.containsSpreadExpression(arguments);
         if (containsSpreadExpression) {
             controller.getAcg().despreadList(ae.getExpressions(), true);
-            sig += getTypeDescription(Object[].class);
+            sig.append(getTypeDescription(Object[].class));
         } else {
             for (Expression arg : ae.getExpressions()) {
                 arg.visit(controller.getAcg());
                 if (arg instanceof CastExpression) {
                     operandStack.box();
                     controller.getAcg().loadWrapper(arg);
-                    sig += getTypeDescription(Wrapper.class);
+                    sig.append(getTypeDescription(Wrapper.class));
                 } else {
-                    sig += getTypeDescription(operandStack.getTopOperand());
+                    sig.append(getTypeDescription(operandStack.getTopOperand()));
                 }
                 numberOfArguments++;
             }
         }
 
-        sig += ")Ljava/lang/Object;";
+        sig.append(")Ljava/lang/Object;");
         String callSiteName = METHOD.getCallSiteName();
         if (adapter==null) callSiteName = INIT.getCallSiteName();
         int flags = getMethodCallFlags(adapter, safe, containsSpreadExpression);
-        finishIndyCall(BSM, callSiteName, sig, numberOfArguments, methodName, flags);
+        finishIndyCall(BSM, callSiteName, sig.toString(), numberOfArguments, methodName, flags);
     }
     
     private static int getMethodCallFlags(MethodCallerMultiAdapter adapter, boolean safe, boolean spread) {
@@ -214,15 +212,14 @@ public class InvokeDynamicWriter extends InvocationWriter {
     }
 
     private void writeIndyCast(ClassNode sourceType, ClassNode targetType) {
-        StringBuilder sig = new StringBuilder();
-        sig.append('(');
-        sig.append(getTypeDescription(sourceType));
-        sig.append(')');
-        sig.append(getTypeDescription(targetType));
 
+        String sig = "(" +
+                getTypeDescription(sourceType) +
+                ')' +
+                getTypeDescription(targetType);
         controller.getMethodVisitor().visitInvokeDynamicInsn(
                 //TODO: maybe use a different bootstrap method since no arguments are needed here
-                CAST.getCallSiteName(), sig.toString(), BSM, "()", 0);
+                CAST.getCallSiteName(), sig, BSM, "()", 0);
         controller.getOperandStack().replace(targetType);
     }
 
