@@ -26,8 +26,8 @@ import org.objectweb.asm.Opcodes;
 
 import java.io.File;
 import java.io.PrintWriter;
-import java.math.BigDecimal;
 import java.util.Arrays;
+import java.util.Collections;
 import java.util.HashMap;
 import java.util.HashSet;
 import java.util.LinkedHashSet;
@@ -38,16 +38,11 @@ import java.util.Properties;
 import java.util.Set;
 import java.util.StringTokenizer;
 
-import static org.apache.groovy.util.SystemUtil.getBooleanSafe;
 import static org.apache.groovy.util.SystemUtil.getSystemPropertySafe;
+import static org.codehaus.groovy.runtime.StringGroovyMethods.isAtLeast;
 
 /**
  * Compilation control flags and coordination stuff.
- *
- * @author <a href="mailto:cpoirier@dreaming.org">Chris Poirier</a>
- * @author <a href="mailto:blackdrag@gmx.org">Jochen Theodorou</a>
- * @author <a href="mailto:jim@pagesmiths.com">Jim White</a>
- * @author <a href="mailto:cedric.champeau@gmail.com">Cedric Champeau</a>
  */
 public class CompilerConfiguration {
 
@@ -72,6 +67,10 @@ public class CompilerConfiguration {
     public static final String JDK11 = "11";
     /** This (<code>"12"</code>) is the value for targetBytecode to compile for a JDK 12. **/
     public static final String JDK12 = "12";
+    /** This (<code>"13"</code>) is the value for targetBytecode to compile for a JDK 13. **/
+    public static final String JDK13 = "13";
+    /** This (<code>"14"</code>) is the value for targetBytecode to compile for a JDK 14. **/
+    public static final String JDK14 = "14";
 
     /**
      * This constant is for comparing targetBytecode to ensure it is set to JDK 1.5 or later.
@@ -87,7 +86,9 @@ public class CompilerConfiguration {
     @Deprecated
     public static final String PRE_JDK5 = JDK4;
 
-    /** JDK version to bytecode version mapping */
+    /**
+     * JDK version to bytecode version mapping
+     */
     public static final Map<String, Integer> JDK_TO_BYTECODE_VERSION_MAP = Maps.of(
             JDK4, Opcodes.V1_4,
             JDK5, Opcodes.V1_5,
@@ -97,11 +98,14 @@ public class CompilerConfiguration {
             JDK9, Opcodes.V9,
             JDK10, Opcodes.V10,
             JDK11, Opcodes.V11,
-            JDK12, Opcodes.V12
+            JDK12, Opcodes.V12,
+            JDK13, Opcodes.V13,
+            JDK14, Opcodes.V14
     );
 
     private static final String[] EMPTY_STRING_ARRAY = new String[0];
-    /** An array of the valid targetBytecode values **/
+
+    /** An array of the valid targetBytecode values */
     public static final String[] ALLOWED_JDKS = JDK_TO_BYTECODE_VERSION_MAP.keySet().toArray(EMPTY_STRING_ARRAY);
 
     @Deprecated
@@ -112,15 +116,163 @@ public class CompilerConfiguration {
      */
     public static final String DEFAULT_SOURCE_ENCODING = "UTF-8";
 
-    // Static initializers are executed in text order,
-    // therefore we must do this one last!
     /**
      *  A convenience for getting a default configuration.  Do not modify it!
      *  See {@link #CompilerConfiguration(Properties)} for an example on how to
      *  make a suitable copy to modify.  But if you're really starting from a
-     *  default context, then you probably just want <code>new CompilerConfiguration()</code>. 
+     *  default context, then you probably just want <code>new CompilerConfiguration()</code>.
      */
-    public static final CompilerConfiguration DEFAULT = new CompilerConfiguration();
+    public static final CompilerConfiguration DEFAULT = new CompilerConfiguration() {
+        @Override
+        public List<String> getClasspath() {
+            return Collections.unmodifiableList(super.getClasspath());
+        }
+
+        @Override
+        public List<CompilationCustomizer> getCompilationCustomizers() {
+            return Collections.unmodifiableList(super.getCompilationCustomizers());
+        }
+
+        @Override
+        public Set<String> getDisabledGlobalASTTransformations() {
+            return Collections.emptySet();
+        }
+
+        @Override
+        public Map<String, Object> getJointCompilationOptions() {
+            return null;
+        }
+
+        @Override
+        public Map<String, Boolean> getOptimizationOptions() {
+            return Collections.unmodifiableMap(super.getOptimizationOptions());
+        }
+
+        @Override
+        public Set<String> getScriptExtensions() {
+            return Collections.unmodifiableSet(super.getScriptExtensions());
+        }
+
+        @Override
+        public void setBytecodePostprocessor(BytecodeProcessor bytecodePostprocessor) {
+            throw new UnsupportedOperationException();
+        }
+
+        @Override
+        public void setClasspath(String classpath) {
+            throw new UnsupportedOperationException();
+        }
+
+        @Override
+        public void setClasspathList(List<String> parts) {
+            throw new UnsupportedOperationException();
+        }
+
+        @Override
+        public CompilerConfiguration addCompilationCustomizers(CompilationCustomizer... customizers) {
+            throw new UnsupportedOperationException();
+        }
+
+        @Override
+        public void setDebug(boolean debug) {
+            throw new UnsupportedOperationException();
+        }
+
+        @Override
+        public void setDefaultScriptExtension(String defaultScriptExtension) {
+            throw new UnsupportedOperationException();
+        }
+
+        @Override
+        public void setDisabledGlobalASTTransformations(Set<String> disabledGlobalASTTransformations) {
+            throw new UnsupportedOperationException();
+        }
+
+        @Override
+        public void setJointCompilationOptions(Map<String, Object> options) {
+            throw new UnsupportedOperationException();
+        }
+
+        @Override
+        public void setMinimumRecompilationInterval(int time) {
+            throw new UnsupportedOperationException();
+        }
+
+        @Override
+        public void setOptimizationOptions(Map<String, Boolean> options) {
+            throw new UnsupportedOperationException();
+        }
+
+        @Override
+        public void setOutput(PrintWriter output) {
+            throw new UnsupportedOperationException();
+        }
+
+        @Override
+        public void setParameters(boolean parameters) {
+            throw new UnsupportedOperationException();
+        }
+
+        @Override
+        public void setPluginFactory(ParserPluginFactory pluginFactory) {
+            throw new UnsupportedOperationException();
+        }
+
+        @Override
+        public void setRecompileGroovySource(boolean recompile) {
+            throw new UnsupportedOperationException();
+        }
+
+        @Override
+        public void setScriptBaseClass(String scriptBaseClass) {
+            throw new UnsupportedOperationException();
+        }
+
+        @Override
+        public void setScriptExtensions(Set<String> scriptExtensions) {
+            throw new UnsupportedOperationException();
+        }
+
+        @Override
+        public void setSourceEncoding(String encoding) {
+            throw new UnsupportedOperationException();
+        }
+
+        @Override
+        public void setTargetBytecode(String version) {
+            throw new UnsupportedOperationException();
+        }
+
+        @Override
+        public void setTargetDirectory(File directory) {
+            throw new UnsupportedOperationException();
+        }
+
+        @Override
+        public void setTargetDirectory(String directory) {
+            throw new UnsupportedOperationException();
+        }
+
+        @Override
+        public void setTolerance(int tolerance) {
+            throw new UnsupportedOperationException();
+        }
+
+        @Override
+        public void setVerbose(boolean verbose) {
+            throw new UnsupportedOperationException();
+        }
+
+        @Override
+        public void setPreviewFeatures(boolean previewFeatures) {
+            throw new UnsupportedOperationException();
+        }
+
+        @Override
+        public void setWarningLevel(int level) {
+            throw new UnsupportedOperationException();
+        }
+    };
 
 
     /**
@@ -134,9 +286,9 @@ public class CompilerConfiguration {
     private String sourceEncoding;
 
     /**
-      * The <code>PrintWriter</code> does nothing.
-      */
-     private PrintWriter output;
+     * The <code>PrintWriter</code> does nothing.
+     */
+    private PrintWriter output;
 
     /**
      * Directory into which to write classes
@@ -200,6 +352,11 @@ public class CompilerConfiguration {
      */
     private String targetBytecode;
 
+    /**
+     * Whether the bytecode version has preview features enabled (JEP 12)
+     */
+    private boolean previewFeatures;
+
     /**
      * options for joint compilation (null by default == no joint compilation)
      */
@@ -224,73 +381,85 @@ public class CompilerConfiguration {
     public static final int ASM_API_VERSION = Opcodes.ASM7;
 
     /**
-     * Sets the Flags to defaults.
+     * Sets the compiler flags/settings to default values.
+     *
+     * The following system properties are referenced when setting the configuration:
+     *
+     * <blockquote>
+     * <table summary="Groovy Compiler Configuration Properties">
+     *   <tr><th>Property Key</th><th>Related Property Getter</th></tr>
+     *   <tr><td><code>groovy.source.encoding</code> (defaulting to <code>file.encoding</code>)</td><td>{@link #getSourceEncoding}</td></tr>
+     *   <tr><td><code>groovy.target.bytecode</code></td><td>{@link #getTargetBytecode}</td></tr>
+     *   <tr><td><code>groovy.target.directory</code></td><td>{@link #getTargetDirectory}</td></tr>
+     *   <tr><td><code>groovy.parameters</code></td><td>{@link #getParameters()}</td></tr>
+     *   <tr><td><code>groovy.preview.features</code></td><td>{@link #isPreviewFeatures}</td></tr>
+     *   <tr><td><code>groovy.script.base</code></td><td>{@link #getScriptBaseClass}</td></tr>
+     *   <tr><td><code>groovy.default.scriptExtension</code></td><td>{@link #getDefaultScriptExtension}</td></tr>
+     * </table>
+     * </blockquote>
+     *
+     * The following system properties are referenced when setting the configuration optimization options:
+     *
+     * <blockquote>
+     * <table summary="Groovy Compiler Optimization Options Configuration Properties">
+     *   <tr><th>Property Key</th><th>Related Property Getter</th></tr>
+     *   <tr><td><code>groovy.target.indy</code></td><td>{@link #getOptimizationOptions}</td></tr>
+     * </table>
+     * </blockquote>
      */
     public CompilerConfiguration() {
-        //
         // Set in safe defaults
+        warningLevel = WarningMessage.LIKELY_ERRORS;
+        classpath = new LinkedList<String>();
+        parameters = getSystemPropertySafe("groovy.parameters") != null;
+        tolerance = 10;
+        minimumRecompilationInterval = 100;
 
-        setWarningLevel(WarningMessage.LIKELY_ERRORS);
-        setOutput(null);
-        setTargetDirectory((File) null);
-        setClasspath("");
-        setVerbose(false);
-        setDebug(false);
-        setParameters(getSystemPropertySafe("groovy.parameters") != null);
-        setTolerance(10);
-        setScriptBaseClass(null);
-        setRecompileGroovySource(false);
-        setMinimumRecompilationInterval(100);
-        setTargetBytecode(getSystemPropertySafe("groovy.target.bytecode", getMinBytecodeVersion()));
-        setDefaultScriptExtension(getSystemPropertySafe("groovy.default.scriptExtension", ".groovy"));
+        setTargetBytecodeIfValid(getSystemPropertySafe("groovy.target.bytecode", getMinBytecodeVersion()));
+
+        previewFeatures = getSystemPropertySafe("groovy.preview.features") != null;
+        defaultScriptExtension = getSystemPropertySafe("groovy.default.scriptExtension", ".groovy");
 
         // Source file encoding
         String encoding = getSystemPropertySafe("file.encoding", DEFAULT_SOURCE_ENCODING);
         encoding = getSystemPropertySafe("groovy.source.encoding", encoding);
-        setSourceEncoding(encoding);
+        setSourceEncodingOrDefault(encoding);
 
-        try {
-            setOutput(new PrintWriter(System.err));
-        } catch (Exception e) {
-            // IGNORE
-        }
+        setTargetDirectorySafe(getSystemPropertySafe("groovy.target.directory"));
 
+        optimizationOptions = new HashMap<>(4);
+        handleOptimizationOption(optimizationOptions, INVOKEDYNAMIC, "groovy.target.indy");
+    }
 
-        String target = getSystemPropertySafe("groovy.target.directory");
-        if (target != null) {
-            setTargetDirectory(target);
-        }
+    private void handleOptimizationOption(Map<String, Boolean> options, String optionName, String sysOptionName) {
+        String propValue = getSystemPropertySafe(sysOptionName);
+        boolean optionEnabled = propValue == null
+                ? (DEFAULT == null ? false : Boolean.TRUE.equals(DEFAULT.getOptimizationOptions().get(optionName)))
+                : Boolean.valueOf(propValue);
 
-        boolean indy = getBooleanSafe("groovy.target.indy");
-        if (DEFAULT!=null && Boolean.TRUE.equals(DEFAULT.getOptimizationOptions().get(INVOKEDYNAMIC))) {
-            indy = true;
+        if (optionEnabled) {
+            options.put(optionName, Boolean.TRUE);
         }
-        Map options = new HashMap<String,Boolean>(3);
-        if (indy) {
-            options.put(INVOKEDYNAMIC, Boolean.TRUE);
-        }
-        setOptimizationOptions(options);
     }
 
     /**
-     * Copy constructor.  Use this if you have a mostly correct configuration
-     * for your compilation but you want to make a some changes programatically.
+     * Copy constructor. Use this if you have a mostly correct configuration
+     * for your compilation but you want to make a some changes programmatically.
      * An important reason to prefer this approach is that your code will most
      * likely be forward compatible with future changes to this configuration API.
      * <p>
      * An example of this copy constructor at work:
-     * <pre>
-     *    // In all likelihood there is already a configuration in your code's context
-     *    // for you to copy, but for the sake of this example we'll use the global default.
-     *    CompilerConfiguration myConfiguration = new CompilerConfiguration(CompilerConfiguration.DEFAULT);
-     *    myConfiguration.setDebug(true);
-     *</pre>
+     * <blockquote><pre>
+     * // In all likelihood there is already a configuration in your code's context
+     * // for you to copy, but for the sake of this example we'll use the global default.
+     * CompilerConfiguration myConfiguration = new CompilerConfiguration(CompilerConfiguration.DEFAULT);
+     * myConfiguration.setDebug(true);
+     * </pre></blockquote>
      *
      * @param configuration The configuration to copy.
      */
     public CompilerConfiguration(CompilerConfiguration configuration) {
         setWarningLevel(configuration.getWarningLevel());
-        setOutput(configuration.getOutput());
         setTargetDirectory(configuration.getTargetDirectory());
         setClasspathList(new LinkedList<String>(configuration.getClasspath()));
         setVerbose(configuration.getVerbose());
@@ -301,66 +470,71 @@ public class CompilerConfiguration {
         setRecompileGroovySource(configuration.getRecompileGroovySource());
         setMinimumRecompilationInterval(configuration.getMinimumRecompilationInterval());
         setTargetBytecode(configuration.getTargetBytecode());
+        setPreviewFeatures(configuration.isPreviewFeatures());
         setDefaultScriptExtension(configuration.getDefaultScriptExtension());
         setSourceEncoding(configuration.getSourceEncoding());
-        setTargetDirectory(configuration.getTargetDirectory());
         Map<String, Object> jointCompilationOptions = configuration.getJointCompilationOptions();
         if (jointCompilationOptions != null) {
             jointCompilationOptions = new HashMap<String, Object>(jointCompilationOptions);
         }
+        // TODO GROOVY-9585: add line below once gradle build issues fixed
+//        compilationCustomizers.addAll(configuration.getCompilationCustomizers());
         setJointCompilationOptions(jointCompilationOptions);
         setPluginFactory(configuration.getPluginFactory());
-        setScriptExtensions(configuration.getScriptExtensions());
+        setDisabledGlobalASTTransformations(configuration.getDisabledGlobalASTTransformations());
+        setScriptExtensions(new LinkedHashSet<String>(configuration.getScriptExtensions()));
         setOptimizationOptions(new HashMap<String, Boolean>(configuration.getOptimizationOptions()));
+        setBytecodePostprocessor(configuration.getBytecodePostprocessor());
     }
 
     /**
-     * Sets the Flags to the specified configuration, with defaults
-     * for those not supplied.
-     * Note that those "defaults" here do <em>not</em> include checking the
-     * settings in {@link System#getProperties()} in general, only file.encoding, 
-     * groovy.target.directory and groovy.source.encoding are.
-     * <p>
+     * Sets the configuration flags/settings according to values from the supplied {@code Properties} instance
+     * or if not found, supplying a default value.
+     *
+     * Note that unlike {@link #CompilerConfiguration()}, the "defaults" here do <em>not</em> in general
+     * include checking the settings in {@link System#getProperties()}.
      * If you want to set a few flags but keep Groovy's default
      * configuration behavior then be sure to make your settings in
-     * a Properties that is backed by <code>System.getProperties()</code> (which
+     * a {@code Properties} object that is backed by <code>System.getProperties()</code> (which
      * is done using this constructor). That might be done like this:
-     * <pre>
+     * <blockquote><pre>
      * Properties myProperties = new Properties(System.getProperties());
      * myProperties.setProperty("groovy.output.debug", "true");
      * myConfiguration = new CompilerConfiguration(myProperties);
-     * </pre>
-     * And you also have to contend with a possible SecurityException when
-     * getting the system properties (See {@link java.lang.System#getProperties()}).
+     * </pre></blockquote>
+     * And you also have to contend with a possible {@code SecurityException} when
+     * getting the system properties (See {@link System#getProperties()}).
      * A safer approach would be to copy a default
-     * CompilerConfiguration and make your changes there using the setter:
-     * <pre>
+     * {@code CompilerConfiguration} and make your changes there using the setter:
+     * <blockquote><pre>
      * // In all likelihood there is already a configuration for you to copy,
      * // but for the sake of this example we'll use the global default.
      * CompilerConfiguration myConfiguration = new CompilerConfiguration(CompilerConfiguration.DEFAULT);
      * myConfiguration.setDebug(true);
-     * </pre>
-     * <p>
+     * </pre></blockquote>
+     *
+     * The following properties are referenced when setting the configuration:
+     *
+     * <blockquote>
      * <table summary="Groovy Compiler Configuration Properties">
-     *   <tr>
-     *      <th>Property Key</th><th>Get/Set Property Name</th>
-     *   </tr>
-     *      <tr>
-     *      <td><code>"groovy.warnings"</code></td><td>{@link #getWarningLevel}</td></tr>
-     *      <tr><td><code>"groovy.source.encoding"</code></td><td>{@link #getSourceEncoding}</td></tr>
-     *      <tr><td><code>"groovy.target.directory"</code></td><td>{@link #getTargetDirectory}</td></tr>
-     *      <tr><td><code>"groovy.target.bytecode"</code></td><td>{@link #getTargetBytecode}</td></tr>
-     *      <tr><td><code>"groovy.classpath"</code></td><td>{@link #getClasspath}</td></tr>
-     *      <tr><td><code>"groovy.output.verbose"</code></td><td>{@link #getVerbose}</td></tr>
-     *      <tr><td><code>"groovy.output.debug"</code></td><td>{@link #getDebug}</td></tr>
-     *      <tr><td><code>"groovy.errors.tolerance"</code></td><td>{@link #getTolerance}</td></tr>
-     *      <tr><td><code>"groovy.script.extension"</code></td><td>{@link #getDefaultScriptExtension}</td></tr>
-     *      <tr><td><code>"groovy.script.base"</code></td><td>{@link #getScriptBaseClass}</td></tr>
-     *      <tr><td><code>"groovy.recompile"</code></td><td>{@link #getRecompileGroovySource}</td></tr>
-     *      <tr><td><code>"groovy.recompile.minimumInterval"</code></td><td>{@link #getMinimumRecompilationInterval}</td></tr>
-     *      <tr><td>
-     *   </tr>
+     *   <tr><th>Property Key</th><th>Related Property Getter</th></tr>
+     *   <tr><td><code>groovy.warnings</code></td><td>{@link #getWarningLevel}</td></tr>
+     *   <tr><td><code>groovy.source.encoding</code> (defaulting to <code>file.encoding</code>)</td><td>{@link #getSourceEncoding}</td></tr>
+     *   <tr><td><code>groovy.target.directory</code></td><td>{@link #getTargetDirectory}</td></tr>
+     *   <tr><td><code>groovy.target.bytecode</code></td><td>{@link #getTargetBytecode}</td></tr>
+     *   <tr><td><code>groovy.parameters</code></td><td>{@link #getParameters()}</td></tr>
+     *   <tr><td><code>groovy.preview.features</code></td><td>{@link #isPreviewFeatures}</td></tr>
+     *   <tr><td><code>groovy.classpath</code></td><td>{@link #getClasspath}</td></tr>
+     *   <tr><td><code>groovy.output.verbose</code></td><td>{@link #getVerbose}</td></tr>
+     *   <tr><td><code>groovy.output.debug</code></td><td>{@link #getDebug}</td></tr>
+     *   <tr><td><code>groovy.errors.tolerance</code></td><td>{@link #getTolerance}</td></tr>
+     *   <tr><td><code>groovy.default.scriptExtension</code></td><td>{@link #getDefaultScriptExtension}</td></tr>
+     *   <tr><td><code>groovy.script.base</code></td><td>{@link #getScriptBaseClass}</td></tr>
+     *   <tr><td><code>groovy.recompile</code></td><td>{@link #getRecompileGroovySource}</td></tr>
+     *   <tr><td><code>groovy.recompile.minimumInterval</code></td><td>{@link #getMinimumRecompilationInterval}</td></tr>
+     *   <tr><td><code>groovy.disabled.global.ast.transformations</code></td><td>{@link #getDisabledGlobalASTTransformations}</td></tr>
      * </table>
+     * </blockquote>
      *
      * @param configuration The properties to get flag values from.
      */
@@ -375,8 +549,8 @@ public class CompilerConfiguration {
      * @param bytecodeVersion The parameter can take one of the values in {@link #ALLOWED_JDKS}.
      * @return true if the bytecode version is JDK 1.5+
      */
-    public static boolean isPostJDK5(String bytecodeVersion) {
-        return new BigDecimal(bytecodeVersion).compareTo(new BigDecimal(JDK5)) >= 0;
+    public static boolean isPostJDK5(final String bytecodeVersion) {
+        return isAtLeast(bytecodeVersion, JDK5);
     }
 
     /**
@@ -385,8 +559,8 @@ public class CompilerConfiguration {
      * @param bytecodeVersion The parameter can take one of the values in {@link #ALLOWED_JDKS}.
      * @return true if the bytecode version is JDK 1.7+
      */
-    public static boolean isPostJDK7(String bytecodeVersion) {
-        return new BigDecimal(bytecodeVersion).compareTo(new BigDecimal(JDK7)) >= 0;
+    public static boolean isPostJDK7(final String bytecodeVersion) {
+        return isAtLeast(bytecodeVersion, JDK7);
     }
 
     /**
@@ -395,8 +569,8 @@ public class CompilerConfiguration {
      * @param bytecodeVersion The parameter can take one of the values in {@link #ALLOWED_JDKS}.
      * @return true if the bytecode version is JDK 1.8+
      */
-    public static boolean isPostJDK8(String bytecodeVersion) {
-        return new BigDecimal(bytecodeVersion).compareTo(new BigDecimal(JDK8)) >= 0;
+    public static boolean isPostJDK8(final String bytecodeVersion) {
+        return isAtLeast(bytecodeVersion, JDK8);
     }
 
     /**
@@ -405,8 +579,8 @@ public class CompilerConfiguration {
      * @param bytecodeVersion The parameter can take one of the values in {@link #ALLOWED_JDKS}.
      * @return true if the bytecode version is JDK 9.0+
      */
-    public static boolean isPostJDK9(String bytecodeVersion) {
-        return new BigDecimal(bytecodeVersion).compareTo(new BigDecimal(JDK9)) >= 0;
+    public static boolean isPostJDK9(final String bytecodeVersion) {
+        return isAtLeast(bytecodeVersion, JDK9);
     }
 
     /**
@@ -415,123 +589,91 @@ public class CompilerConfiguration {
      * @param configuration The properties to get flag values from.
      */
     public void configure(Properties configuration) throws ConfigurationException {
-        String text = null;
-        int numeric = 0;
-
-        //
-        // Warning level
+        String text;
+        int numeric;
 
         numeric = getWarningLevel();
+        text = configuration.getProperty("groovy.warnings", "likely errors");
         try {
-            text = configuration.getProperty("groovy.warnings", "likely errors");
             numeric = Integer.parseInt(text);
         } catch (NumberFormatException e) {
             text = text.toLowerCase();
             if (text.equals("none")) {
                 numeric = WarningMessage.NONE;
-            }
-            else if (text.startsWith("likely")) {
+            } else if (text.startsWith("likely")) {
                 numeric = WarningMessage.LIKELY_ERRORS;
-            }
-            else if (text.startsWith("possible")) {
+            } else if (text.startsWith("possible")) {
                 numeric = WarningMessage.POSSIBLE_ERRORS;
-            }
-            else if (text.startsWith("paranoia")) {
+            } else if (text.startsWith("paranoia")) {
                 numeric = WarningMessage.PARANOIA;
-            }
-            else {
+            } else {
                 throw new ConfigurationException("unrecognized groovy.warnings: " + text);
             }
         }
         setWarningLevel(numeric);
 
-        //
-        // Source file encoding
-        //
         text = configuration.getProperty("groovy.source.encoding");
         if (text == null) {
             text = configuration.getProperty("file.encoding", DEFAULT_SOURCE_ENCODING);
         }
         setSourceEncoding(text);
 
-        //
-        // Target directory for classes
-        //
         text = configuration.getProperty("groovy.target.directory");
         if (text != null) setTargetDirectory(text);
 
         text = configuration.getProperty("groovy.target.bytecode");
         if (text != null) setTargetBytecode(text);
 
-        //
-        // Classpath
-        //
+        text = configuration.getProperty("groovy.parameters");
+        if (text != null) setParameters(text.equalsIgnoreCase("true"));
+
+        text = configuration.getProperty("groovy.preview.features");
+        if (text != null) setPreviewFeatures(text.equalsIgnoreCase("true"));
+
         text = configuration.getProperty("groovy.classpath");
         if (text != null) setClasspath(text);
 
-        //
-        // Verbosity
-        //
         text = configuration.getProperty("groovy.output.verbose");
-        if (text != null && text.equalsIgnoreCase("true")) setVerbose(true);
+        if (text != null) setVerbose(text.equalsIgnoreCase("true"));
 
-        //
-        // Debugging
-        //
         text = configuration.getProperty("groovy.output.debug");
-        if (text != null && text.equalsIgnoreCase("true")) setDebug(true);
+        if (text != null) setDebug(text.equalsIgnoreCase("true"));
 
-        //
-        // Parameters
-        //
-        setParameters(configuration.getProperty("groovy.parameters") != null);
-
-        //
-        // Tolerance
-        //
         numeric = 10;
+        text = configuration.getProperty("groovy.errors.tolerance", "10");
         try {
-            text = configuration.getProperty("groovy.errors.tolerance", "10");
             numeric = Integer.parseInt(text);
         } catch (NumberFormatException e) {
             throw new ConfigurationException(e);
         }
         setTolerance(numeric);
 
-        //
-        // Script Base Class
-        //
+        text = configuration.getProperty("groovy.default.scriptExtension");
+        if (text != null) setDefaultScriptExtension(text);
+
         text = configuration.getProperty("groovy.script.base");
-        if (text!=null) setScriptBaseClass(text);
+        if (text != null) setScriptBaseClass(text);
 
-        //
-        // recompilation options
-        //
         text = configuration.getProperty("groovy.recompile");
-        if (text != null) {
-            setRecompileGroovySource(text.equalsIgnoreCase("true"));
-        }
+        if (text != null) setRecompileGroovySource(text.equalsIgnoreCase("true"));
 
         numeric = 100;
+        text = configuration.getProperty("groovy.recompile.minimumIntervall"); // legacy misspelling
         try {
-            text = configuration.getProperty("groovy.recompile.minimumIntervall");
-            if (text==null) text = configuration.getProperty("groovy.recompile.minimumInterval");
-            if (text!=null) {
+            if (text == null) text = configuration.getProperty("groovy.recompile.minimumInterval");
+            if (text != null) {
                 numeric = Integer.parseInt(text);
-            } else {
-                numeric = 100;
             }
         } catch (NumberFormatException e) {
             throw new ConfigurationException(e);
         }
         setMinimumRecompilationInterval(numeric);
 
-        // disabled global AST transformations
         text = configuration.getProperty("groovy.disabled.global.ast.transformations");
-        if (text!=null) {
+        if (text != null) {
             String[] classNames = text.split(",\\s*}");
-            Set<String> blacklist = new HashSet<String>(Arrays.asList(classNames));
-            setDisabledGlobalASTTransformations(blacklist);
+            Set<String> disabledTransforms = new HashSet<String>(Arrays.asList(classNames));
+            setDisabledGlobalASTTransformations(disabledTransforms);
         }
     }
 
@@ -566,6 +708,10 @@ public class CompilerConfiguration {
      * Sets the encoding to be used when reading source files.
      */
     public void setSourceEncoding(String encoding) {
+        setSourceEncodingOrDefault(encoding);
+    }
+
+    private void setSourceEncodingOrDefault(String encoding) {
         if (encoding == null) encoding = DEFAULT_SOURCE_ENCODING;
         this.sourceEncoding = encoding;
     }
@@ -587,8 +733,7 @@ public class CompilerConfiguration {
     public void setOutput(PrintWriter output) {
         if (output == null) {
             this.output = new PrintWriter(NullWriter.DEFAULT);
-        }
-        else {
+        } else {
             this.output = output;
         }
     }
@@ -604,6 +749,10 @@ public class CompilerConfiguration {
      * Sets the target directory.
      */
     public void setTargetDirectory(String directory) {
+        setTargetDirectorySafe(directory);
+    }
+
+    private void setTargetDirectorySafe(String directory) {
         if (directory != null && directory.length() > 0) {
             this.targetDirectory = new File(directory);
         } else {
@@ -665,13 +814,6 @@ public class CompilerConfiguration {
         return this.debug;
     }
 
-    /**
-     * Returns true if parameter metadata generation has been enabled.
-     */
-    public boolean getParameters() {
-        return this.parameters;
-    }
-
     /**
      * Turns debugging operation on or off.
      */
@@ -679,6 +821,13 @@ public class CompilerConfiguration {
         this.debug = debug;
     }
 
+    /**
+     * Returns true if parameter metadata generation has been enabled.
+     */
+    public boolean getParameters() {
+        return this.parameters;
+    }
+
     /**
      * Turns parameter metadata generation on or off.
      */
@@ -780,6 +929,10 @@ public class CompilerConfiguration {
      * @param version the bytecode compatibility level
      */
     public void setTargetBytecode(String version) {
+        setTargetBytecodeIfValid(version);
+    }
+
+    private void setTargetBytecodeIfValid(String version) {
         if (JDK_TO_BYTECODE_VERSION_MAP.keySet().contains(version)) {
             this.targetBytecode = version;
         }
@@ -796,6 +949,24 @@ public class CompilerConfiguration {
         return this.targetBytecode;
     }
 
+    /**
+     * Whether the bytecode version has preview features enabled (JEP 12)
+     *
+     * @return preview features
+     */
+    public boolean isPreviewFeatures() {
+        return previewFeatures;
+    }
+
+    /**
+     * Sets whether the bytecode version has preview features enabled (JEP 12).
+     *
+     * @param previewFeatures whether to support preview features
+     */
+    public void setPreviewFeatures(boolean previewFeatures) {
+        this.previewFeatures = previewFeatures;
+    }
+
     private static String getMinBytecodeVersion() {
         return JDK7;
     }
@@ -834,7 +1005,7 @@ public class CompilerConfiguration {
      * @throws IllegalArgumentException if the options are null
      */
     public void setOptimizationOptions(Map<String, Boolean> options) {
-        if (options==null) throw new IllegalArgumentException("provided option map must not be null");
+        if (options == null) throw new IllegalArgumentException("provided option map must not be null");
         optimizationOptions = options;
     }
 
@@ -845,7 +1016,7 @@ public class CompilerConfiguration {
      * @return this configuration instance
      */
     public CompilerConfiguration addCompilationCustomizers(CompilationCustomizer... customizers) {
-        if (customizers==null) throw new IllegalArgumentException("provided customizers list must not be null");
+        if (customizers == null) throw new IllegalArgumentException("provided customizers list must not be null");
         compilationCustomizers.addAll(Arrays.asList(customizers));
         return this;
     }
@@ -867,14 +1038,15 @@ public class CompilerConfiguration {
     }
 
     /**
-     * Disables the specified global AST transformations. In order to avoid class loading side effects, it is not recommended
-     * to use MyASTTransformation.class.getName() by directly use the class name as a string. Disabled AST transformations
-     * only apply to automatically loaded global AST transformations, that is to say transformations defined in a
+     * Disables the specified global AST transformations. In order to avoid class loading side effects,
+     * it is not recommended to use MyASTTransformation.class.getName() but instead directly use the class
+     * name as a string. Disabled AST transformations only apply to automatically loaded global AST
+     * transformations, that is to say transformations defined in a
      * META-INF/services/org.codehaus.groovy.transform.ASTTransformation file.
-     * If you explicitly add a global AST transformation
-     * in your compilation process, for example using the {@link org.codehaus.groovy.control.customizers.ASTTransformationCustomizer} or
-     * using a {@link org.codehaus.groovy.control.CompilationUnit.PrimaryClassNodeOperation}, then nothing will prevent
-     * the transformation from being loaded.
+     * If you explicitly add a global AST transformation in your compilation process,
+     * for example using the {@link org.codehaus.groovy.control.customizers.ASTTransformationCustomizer} or
+     * using a {@link org.codehaus.groovy.control.CompilationUnit.PrimaryClassNodeOperation},
+     * then nothing will prevent the transformation from being loaded.
      *
      * @param disabledGlobalASTTransformations a set of fully qualified class names of global AST transformations
      * which should not be loaded.
