@@ -18,19 +18,18 @@
  */
 package org.codehaus.groovy.runtime.metaclass;
 
-import java.util.Map;
+import java.io.IOException;
+import java.io.Serializable;
+import java.util.AbstractCollection;
 import java.util.AbstractMap;
 import java.util.AbstractSet;
-import java.util.AbstractCollection;
-import java.util.Collection;
-import java.util.Set;
 import java.util.ArrayList;
-import java.util.Iterator;
+import java.util.Collection;
 import java.util.Enumeration;
+import java.util.Iterator;
+import java.util.Map;
 import java.util.NoSuchElementException;
-
-import java.io.Serializable;
-import java.io.IOException;
+import java.util.Set;
 
 /**
  * A hash table that supports mostly-concurrent reading, but
@@ -140,6 +139,7 @@ import java.io.IOException;
 public class ConcurrentReaderHashMap 
   extends AbstractMap 
   implements Cloneable, Serializable {
+  private static final long serialVersionUID = -3225682440765612861L;
 
 
   /*
@@ -166,7 +166,9 @@ public class ConcurrentReaderHashMap
   */
 
   /** A Serializable class for barrier lock **/
-  protected static class BarrierLock implements java.io.Serializable { }
+  protected static class BarrierLock implements java.io.Serializable {
+      private static final long serialVersionUID = -2159505361622844863L;
+  }
 
   /**
    * Lock used only for its memory effects.
@@ -176,7 +178,6 @@ public class ConcurrentReaderHashMap
   /**
    * field written to only to guarantee lock ordering.
    **/
-
   protected transient Object lastWrite;
 
   /**
