@@ -19,6 +19,7 @@
 package org.codehaus.groovy.tools;
 
 import org.apache.groovy.util.SystemUtil;
+import org.codehaus.groovy.runtime.DefaultGroovyMethods;
 
 import java.io.BufferedReader;
 import java.io.File;
@@ -78,8 +79,8 @@ import java.util.regex.Pattern;
  */
 public class LoaderConfiguration {
 
-    private static final String MAIN_PREFIX = "main is", LOAD_PREFIX = "load", GRAB_PREFIX = "grab", PROP_PREFIX = "property";
-    private List<URL> classPath = new ArrayList<URL>();
+    private static final String MAIN_PREFIX = "main is", LOAD_PREFIX = "load", GRAB_PREFIX = "grab", PROP_PREFIX = "property", CONFIGSCRIPT_PREFIX = "configscript";
+    private final List<URL> classPath = new ArrayList<URL>();
     private String main;
     private boolean requireMain;
     private static final char WILDCARD = '*';
@@ -87,6 +88,7 @@ public class LoaderConfiguration {
     private static final String MATCH_FILE_NAME = "\\\\E[^/]+?\\\\Q";
     private static final String MATCH_ALL = "\\\\E.+?\\\\Q";
     private final List<String> grabList = new ArrayList<String>();
+    private final List<String> configScripts = new ArrayList<String>();
 
     /**
      * creates a new loader configuration
@@ -102,9 +104,7 @@ public class LoaderConfiguration {
      * @throws IOException if reading or parsing the contents of the stream fails
      */
     public void configure(InputStream is) throws IOException {
-        BufferedReader reader = new BufferedReader(new InputStreamReader(is));
-        try {
-
+        try (BufferedReader reader = new BufferedReader(new InputStreamReader(is))) {
             int lineNumber = 0;
 
             while (true) {
@@ -131,15 +131,19 @@ public class LoaderConfiguration {
                     String params = line.substring(PROP_PREFIX.length()).trim();
                     String key = SystemUtil.setSystemPropertyFrom(params);
                     System.setProperty(key, assignProperties(System.getProperty(key)));
+                } else if (line.startsWith(CONFIGSCRIPT_PREFIX)) {
+                    String script = line.substring(CONFIGSCRIPT_PREFIX.length()).trim();
+                    configScripts.add(assignProperties(script));
                 } else {
                     throw new IOException("unexpected line in " + lineNumber + " : " + line);
                 }
             }
-        } finally {
-            reader.close();
         }
 
         if (requireMain && main == null) throw new IOException("missing main class definition in config file");
+        if (!configScripts.isEmpty()) {
+            System.setProperty("groovy.starter.configscripts", DefaultGroovyMethods.join((Iterable)configScripts, ","));
+        }
     }
 
     /*
@@ -164,7 +168,7 @@ public class LoaderConfiguration {
                 requireProperty = propertyIndexStart == i2;
             }
             if (propertyIndexStart == -1) break;
-            result.append(str.substring(propertyIndexEnd, propertyIndexStart));
+            result.append(str, propertyIndexEnd, propertyIndexStart);
 
             propertyIndexEnd = str.indexOf("}", propertyIndexStart);
             if (propertyIndexEnd == -1) break;
@@ -322,7 +326,7 @@ public class LoaderConfiguration {
      * @see java.net.URLClassLoader
      */
     public URL[] getClassPathUrls() {
-        return classPath.toArray(new URL[classPath.size()]);
+        return classPath.toArray(new URL[0]);
     }
 
     /**
