@@ -305,15 +305,15 @@ public class Os
             {
                 if ( family.equalsIgnoreCase( FAMILY_WINDOWS ) )
                 {
-                    isFamily = OS_NAME.indexOf( FAMILY_WINDOWS ) > -1;
+                    isFamily = OS_NAME.contains( FAMILY_WINDOWS );
                 }
                 else if ( family.equalsIgnoreCase( FAMILY_OS2 ) )
                 {
-                    isFamily = OS_NAME.indexOf( FAMILY_OS2 ) > -1;
+                    isFamily = OS_NAME.contains( FAMILY_OS2 );
                 }
                 else if ( family.equalsIgnoreCase( FAMILY_NETWARE ) )
                 {
-                    isFamily = OS_NAME.indexOf( FAMILY_NETWARE ) > -1;
+                    isFamily = OS_NAME.contains( FAMILY_NETWARE );
                 }
                 else if ( family.equalsIgnoreCase( FAMILY_DOS ) )
                 {
@@ -321,11 +321,11 @@ public class Os
                 }
                 else if ( family.equalsIgnoreCase( FAMILY_MAC ) )
                 {
-                    isFamily = OS_NAME.indexOf( FAMILY_MAC ) > -1;
+                    isFamily = OS_NAME.contains( FAMILY_MAC );
                 }
                 else if ( family.equalsIgnoreCase( FAMILY_TANDEM ) )
                 {
-                    isFamily = OS_NAME.indexOf( "nonstop_kernel" ) > -1;
+                    isFamily = OS_NAME.contains( "nonstop_kernel" );
                 }
                 else if ( family.equalsIgnoreCase( FAMILY_UNIX ) )
                 {
@@ -335,24 +335,24 @@ public class Os
                 else if ( family.equalsIgnoreCase( FAMILY_WIN9X ) )
                 {
                     isFamily = isFamily( FAMILY_WINDOWS )
-                        && ( OS_NAME.indexOf( "95" ) >= 0 || OS_NAME.indexOf( "98" ) >= 0
-                            || OS_NAME.indexOf( "me" ) >= 0 || OS_NAME.indexOf( "ce" ) >= 0 );
+                        && ( OS_NAME.contains( "95" ) || OS_NAME.contains( "98" )
+                            || OS_NAME.contains( "me" ) || OS_NAME.contains( "ce" ) );
                 }
                 else if ( family.equalsIgnoreCase( FAMILY_ZOS ) )
                 {
-                    isFamily = OS_NAME.indexOf( FAMILY_ZOS ) > -1 || OS_NAME.indexOf( "os/390" ) > -1;
+                    isFamily = OS_NAME.contains( FAMILY_ZOS ) || OS_NAME.contains( "os/390" );
                 }
                 else if ( family.equalsIgnoreCase( FAMILY_OS400 ) )
                 {
-                    isFamily = OS_NAME.indexOf( FAMILY_OS400 ) > -1;
+                    isFamily = OS_NAME.contains( FAMILY_OS400 );
                 }
                 else if ( family.equalsIgnoreCase( FAMILY_OPENVMS ) )
                 {
-                    isFamily = OS_NAME.indexOf( FAMILY_OPENVMS ) > -1;
+                    isFamily = OS_NAME.contains( FAMILY_OPENVMS );
                 }
                 else
                 {
-                    isFamily = OS_NAME.indexOf( family.toLowerCase( Locale.US ) ) > -1;
+                    isFamily = OS_NAME.contains( family.toLowerCase( Locale.US ) );
                 }
             }
             if ( name != null )
