@@ -17,6 +17,7 @@ package org.gradle.api.internal.artifacts.dsl.dependencies;
 
 import groovy.lang.Closure;
 import org.gradle.api.artifacts.ClientModule;
+import org.gradle.api.artifacts.DependencyConstraint;
 import org.gradle.api.artifacts.Dependency;
 import org.gradle.api.artifacts.ProjectDependency;
 
@@ -25,10 +26,17 @@ import java.util.Map;
 public interface DependencyFactory {
     //for gradle distribution specific dependencies
     enum ClassPathNotation {
-        GRADLE_API, GRADLE_TEST_KIT, LOCAL_GROOVY
+        GRADLE_API("Gradle API"), GRADLE_TEST_KIT("Gradle TestKit"), LOCAL_GROOVY("Local Groovy");
+
+        public final String displayName;
+
+        ClassPathNotation(String displayName) {
+            this.displayName = displayName;
+        }
     }
 
-    Dependency createDependency(Object dependencyNotation);
+    Dependency createDependency(Object dependencyNotation); //we should consider to change the return type to DirectDependency, which requires adjustment in Kotlin DSL
+    DependencyConstraint createDependencyConstraint(Object dependencyNotation);
     ClientModule createModule(Object dependencyNotation, Closure configureClosure);
     ProjectDependency createProjectDependencyFromMap(ProjectFinder projectFinder, Map<? extends String, ? extends Object> map);
 }
