@@ -15,17 +15,25 @@
  */
 package org.gradle.api.internal.changedetection.changes;
 
+import com.google.common.collect.ImmutableSortedMap;
 import org.gradle.StartParameter;
 import org.gradle.api.internal.TaskExecutionHistory;
 import org.gradle.api.internal.TaskInternal;
 import org.gradle.api.internal.changedetection.TaskArtifactState;
 import org.gradle.api.internal.changedetection.TaskArtifactStateRepository;
-import org.gradle.api.internal.changedetection.state.FilesSnapshotSet;
-import org.gradle.api.internal.tasks.cache.TaskCacheKey;
+import org.gradle.api.internal.tasks.OriginTaskExecutionMetadata;
+import org.gradle.api.internal.tasks.TaskExecutionContext;
+import org.gradle.api.internal.tasks.execution.TaskProperties;
+import org.gradle.api.specs.AndSpec;
 import org.gradle.api.tasks.incremental.IncrementalTaskInputs;
+import org.gradle.caching.internal.tasks.TaskOutputCachingBuildCacheKey;
+import org.gradle.internal.fingerprint.CurrentFileCollectionFingerprint;
+import org.gradle.internal.fingerprint.FileCollectionFingerprint;
+import org.gradle.internal.id.UniqueId;
 import org.gradle.internal.reflect.Instantiator;
 
 import java.util.Collection;
+import java.util.Map;
 
 public class ShortCircuitTaskArtifactStateRepository implements TaskArtifactStateRepository {
 
@@ -39,19 +47,26 @@ public class ShortCircuitTaskArtifactStateRepository implements TaskArtifactStat
         this.repository = repository;
     }
 
-    public TaskArtifactState getStateFor(final TaskInternal task) {
+    public TaskArtifactState getStateFor(final TaskInternal task, TaskProperties taskProperties) {
 
-        if (!task.getOutputs().getHasOutput()) { // Only false if no declared outputs AND no Task.upToDateWhen spec. We force to true for incremental tasks.
-            return new NoHistoryArtifactState();
+        // Only false if no declared outputs AND no Task.upToDateWhen spec. We force to true for incremental tasks.
+        AndSpec<? super TaskInternal> upToDateSpec = task.getOutputs().getUpToDateSpec();
+        if (!taskProperties.hasDeclaredOutputs() && upToDateSpec.isEmpty()) {
+            if (task.hasTaskActions()) {
+                return NoOutputsArtifactState.WITH_ACTIONS;
+            } else {
+                return NoOutputsArtifactState.WITHOUT_ACTIONS;
+
+            }
         }
 
-        final TaskArtifactState state = repository.getStateFor(task);
+        TaskArtifactState state = repository.getStateFor(task, taskProperties);
 
         if (startParameter.isRerunTasks()) {
             return new RerunTaskArtifactState(state, task, "Executed with '--rerun-tasks'.");
         }
 
-        if (!task.getOutputs().getUpToDateSpec().isSatisfiedBy(task)) {
+        if (!upToDateSpec.isSatisfiedBy(task)) {
             return new RerunTaskArtifactState(state, task, "Task.upToDateWhen is false.");
         }
 
@@ -69,37 +84,63 @@ public class ShortCircuitTaskArtifactStateRepository implements TaskArtifactStat
             this.reason = reason;
         }
 
+        @Override
         public boolean isUpToDate(Collection<String> messages) {
-            if (messages != null) {
-                messages.add(reason);
-            }
+            // Ensure that we snapshot the task's inputs
+            delegate.ensureSnapshotBeforeTask();
+            messages.add(reason);
             return false;
         }
 
+        @Override
         public IncrementalTaskInputs getInputChanges() {
-            return instantiator.newInstance(RebuildIncrementalTaskInputs.class, task, FilesSnapshotSet.EMPTY);
+            return instantiator.newInstance(RebuildIncrementalTaskInputs.class, task, getCurrentInputFileFingerprints());
         }
 
         @Override
-        public TaskCacheKey calculateCacheKey() {
-            return null;
+        public Iterable<? extends FileCollectionFingerprint> getCurrentInputFileFingerprints() {
+            return delegate.getCurrentInputFileFingerprints();
         }
 
+        @Override
+        public boolean isAllowedToUseCachedResults() {
+            return false;
+        }
+
+        @Override
+        public TaskOutputCachingBuildCacheKey calculateCacheKey() {
+            return delegate.calculateCacheKey();
+        }
+
+        @Override
         public TaskExecutionHistory getExecutionHistory() {
             return delegate.getExecutionHistory();
         }
 
-        public void beforeTask() {
-            delegate.beforeTask();
+        @Override
+        public Map<String, CurrentFileCollectionFingerprint> getOutputFingerprints() {
+            return delegate.getOutputFingerprints();
         }
 
-        public void afterTask() {
-            delegate.afterTask();
+        @Override
+        public void ensureSnapshotBeforeTask() {
+            delegate.ensureSnapshotBeforeTask();
         }
 
-        public void finished() {
-            delegate.finished();
+        @Override
+        public void afterOutputsRemovedBeforeTask() {
+            delegate.afterOutputsRemovedBeforeTask();
+        }
+
+
+        @Override
+        public void snapshotAfterTaskExecution(Throwable failure, UniqueId buildInvocationId, TaskExecutionContext taskExecutionContext) {
+            delegate.snapshotAfterTaskExecution(failure, buildInvocationId, taskExecutionContext);
+        }
+
+        @Override
+        public void snapshotAfterLoadedFromCache(ImmutableSortedMap<String, CurrentFileCollectionFingerprint> newOutputFingerprints, OriginTaskExecutionMetadata originMetadata) {
+            delegate.snapshotAfterLoadedFromCache(newOutputFingerprints, originMetadata);
         }
     }
-
 }
