@@ -17,4 +17,7 @@
 /**
  * The archive bundling {@link org.gradle.api.Task} implementations.
  */
-package org.gradle.api.tasks.bundling;
\ No newline at end of file
+@NonNullApi
+package org.gradle.api.tasks.bundling;
+
+import org.gradle.api.NonNullApi;
