@@ -19,21 +19,29 @@ import com.google.common.io.Files;
 import groovy.lang.Script;
 import org.codehaus.groovy.ast.ClassNode;
 import org.gradle.api.Action;
-import org.gradle.api.internal.changedetection.state.FileSnapshotter;
 import org.gradle.api.internal.initialization.loadercache.ClassLoaderCache;
 import org.gradle.api.internal.initialization.loadercache.ClassLoaderId;
-import org.gradle.api.internal.initialization.loadercache.DefaultClassLoaderCache;
 import org.gradle.cache.CacheRepository;
-import org.gradle.cache.CacheValidator;
 import org.gradle.cache.PersistentCache;
 import org.gradle.groovy.scripts.ScriptSource;
-import org.gradle.initialization.ClassLoaderRegistry;
 import org.gradle.internal.UncheckedException;
-import org.gradle.internal.classloader.ClassLoaderVisitor;
-import org.gradle.logging.ProgressLogger;
-import org.gradle.logging.ProgressLoggerFactory;
+import org.gradle.internal.classanalysis.AsmConstants;
+import org.gradle.internal.classloader.ClassLoaderHierarchyHasher;
+import org.gradle.internal.hash.HashCode;
+import org.gradle.internal.hash.HashUtil;
+import org.gradle.internal.hash.Hashing;
+import org.gradle.internal.logging.progress.ProgressLogger;
+import org.gradle.internal.logging.progress.ProgressLoggerFactory;
 import org.gradle.model.dsl.internal.transform.RuleVisitor;
-import org.objectweb.asm.*;
+import org.objectweb.asm.AnnotationVisitor;
+import org.objectweb.asm.ClassReader;
+import org.objectweb.asm.ClassVisitor;
+import org.objectweb.asm.ClassWriter;
+import org.objectweb.asm.FieldVisitor;
+import org.objectweb.asm.Label;
+import org.objectweb.asm.MethodVisitor;
+import org.objectweb.asm.Opcodes;
+import org.objectweb.asm.Type;
 
 import java.io.Closeable;
 import java.io.File;
@@ -44,24 +52,23 @@ import java.net.URI;
  * A {@link ScriptClassCompiler} which compiles scripts to a cache directory, and loads them from there.
  */
 public class FileCacheBackedScriptClassCompiler implements ScriptClassCompiler, Closeable {
+    private static final Object[] EMPTY_OBJECT_ARRAY = new Object[0];
     private final ScriptCompilationHandler scriptCompilationHandler;
     private final ProgressLoggerFactory progressLoggerFactory;
     private final CacheRepository cacheRepository;
-    private final CacheValidator validator;
-    private final FileSnapshotter snapshotter;
+    private final ScriptSourceHasher hasher;
     private final ClassLoaderCache classLoaderCache;
-    private final ClassLoaderRegistry classLoaderRegistry;
+    private final ClassLoaderHierarchyHasher classLoaderHierarchyHasher;
 
-    public FileCacheBackedScriptClassCompiler(CacheRepository cacheRepository, CacheValidator validator, ScriptCompilationHandler scriptCompilationHandler,
-                                              ProgressLoggerFactory progressLoggerFactory, FileSnapshotter snapshotter, ClassLoaderCache classLoaderCache,
-                                              ClassLoaderRegistry classLoaderRegistry) {
+    public FileCacheBackedScriptClassCompiler(CacheRepository cacheRepository, ScriptCompilationHandler scriptCompilationHandler,
+                                              ProgressLoggerFactory progressLoggerFactory, ScriptSourceHasher hasher, ClassLoaderCache classLoaderCache,
+                                              ClassLoaderHierarchyHasher classLoaderHierarchyHasher) {
         this.cacheRepository = cacheRepository;
-        this.validator = validator;
         this.scriptCompilationHandler = scriptCompilationHandler;
         this.progressLoggerFactory = progressLoggerFactory;
-        this.snapshotter = snapshotter;
+        this.hasher = hasher;
         this.classLoaderCache = classLoaderCache;
-        this.classLoaderRegistry = classLoaderRegistry;
+        this.classLoaderHierarchyHasher = classLoaderHierarchyHasher;
     }
 
     @Override
@@ -76,9 +83,14 @@ public class FileCacheBackedScriptClassCompiler implements ScriptClassCompiler,
             return emptyCompiledScript(classLoaderId, operation);
         }
 
-        final String sourceHash = hashFor(source);
+        HashCode sourceHashCode = hasher.hash(source);
+        final String sourceHash = HashUtil.compactStringFor(sourceHashCode);
         final String dslId = operation.getId();
-        final String classpathHash = dslId + getClassLoaderHash(classLoader);
+        HashCode classLoaderHash = classLoaderHierarchyHasher.getClassLoaderHash(classLoader);
+        if (classLoaderHash == null) {
+            throw new IllegalArgumentException("Unknown classloader: " + classLoader);
+        }
+        final String classpathHash = dslId + classLoaderHash;
         final RemappingScriptSource remapped = new RemappingScriptSource(source);
 
         // Caching involves 2 distinct caches, so that 2 scripts with the same (hash, classpath) do not get compiled twice
@@ -87,70 +99,20 @@ public class FileCacheBackedScriptClassCompiler implements ScriptClassCompiler,
         // Both caches can be closed directly after use because:
         // For 1, if the script changes or its compile classpath changes, a different directory will be used
         // For 2, if the script changes, a different cache is used. If the classpath changes, the cache is invalidated, but classes are remapped to 1. anyway so never directly used
-        PersistentCache remappedClassesCache = cacheRepository.cache(String.format("scripts-remapped/%s/%s/%s", source.getClassName(), sourceHash, classpathHash))
-            .withDisplayName(String.format("%s remapped class cache for %s", dslId, sourceHash))
-            .withValidator(validator)
+        PersistentCache remappedClassesCache = cacheRepository.cache("scripts-remapped/" + source.getClassName() + "/" + sourceHash + "/" + classpathHash)
+            .withDisplayName(dslId + " remapped class cache for " + sourceHash)
             .withInitializer(new ProgressReportingInitializer(progressLoggerFactory, new RemapBuildScriptsAction<M, T>(remapped, classpathHash, sourceHash, dslId, classLoader, operation, verifier, scriptBaseClass),
                 "Compiling script into cache",
-                "Compiling " + source.getFileName() + " into local build cache"))
+                "Compiling " + source.getFileName() + " into local compilation cache"))
             .open();
-        remappedClassesCache.close();
+        try {
+            File remappedClassesDir = classesDir(remappedClassesCache);
+            File remappedMetadataDir = metadataDir(remappedClassesCache);
 
-        File remappedClassesDir = classesDir(remappedClassesCache);
-        File remappedMetadataDir = metadataDir(remappedClassesCache);
-
-        return scriptCompilationHandler.loadFromDir(source, classLoader, remappedClassesDir, remappedMetadataDir, operation, scriptBaseClass, classLoaderId);
-    }
-
-    private int getClassLoaderHash(ClassLoader cl) {
-        ClassloaderHasher hasher = new ClassloaderHasher(classLoaderRegistry);
-        hasher.visit(cl);
-        return hasher.hash;
-    }
-
-    private static class ClassloaderHasher extends ClassLoaderVisitor {
-        private final ClassLoaderRegistry classLoaderRegistry;
-
-        private int hash;
-
-        private ClassloaderHasher(ClassLoaderRegistry classLoaderRegistry) {
-            this.classLoaderRegistry = classLoaderRegistry;
+            return scriptCompilationHandler.loadFromDir(source, sourceHashCode, classLoader, remappedClassesDir, remappedMetadataDir, operation, scriptBaseClass, classLoaderId);
+        } finally {
+            remappedClassesCache.close();
         }
-
-        private int hashFor(ClassLoader cl) {
-            if (classLoaderRegistry.getRuntimeClassLoader() == cl) {
-                return 1;
-            }
-            if (classLoaderRegistry.getGradleApiClassLoader() == cl) {
-                return 2;
-            }
-            if (classLoaderRegistry.getGradleCoreApiClassLoader() == cl) {
-                return 3;
-            }
-            if (classLoaderRegistry.getPluginsClassLoader() == cl) {
-                return 5;
-            }
-            ClassLoader systemClassLoader = ClassLoader.getSystemClassLoader();
-            if (systemClassLoader == cl) {
-                return 7;
-            }
-            if (systemClassLoader != null && systemClassLoader.getParent() == cl) {
-                return 11;
-            }
-            if (cl instanceof DefaultClassLoaderCache.HashedClassLoader) {
-                return ((DefaultClassLoaderCache.HashedClassLoader) cl).getClassLoaderHash();
-            }
-            return 0;
-        }
-
-        public void visit(ClassLoader classLoader) {
-            ClassLoader end = ClassLoader.getSystemClassLoader();
-            if (classLoader != null && classLoader != end) {
-                hash = 31 * hash + hashFor(classLoader);
-            }
-            super.visit(classLoader);
-        }
-
     }
 
     private <T extends Script, M> CompiledScript<T, M> emptyCompiledScript(ClassLoaderId classLoaderId, CompileOperation<M> operation) {
@@ -158,10 +120,6 @@ public class FileCacheBackedScriptClassCompiler implements ScriptClassCompiler,
         return new EmptyCompiledScript<T, M>(operation);
     }
 
-    private String hashFor(ScriptSource source) {
-        return snapshotter.snapshot(source.getResource()).getHash().asCompactString();
-    }
-
     public void close() {
     }
 
@@ -251,15 +209,50 @@ public class FileCacheBackedScriptClassCompiler implements ScriptClassCompiler,
     }
 
     private static class BuildScriptRemapper extends ClassVisitor implements Opcodes {
+        private static final String SCRIPT_ORIGIN = "org/gradle/internal/scripts/ScriptOrigin";
         private final ScriptSource scriptSource;
+        private final String originalClassName;
+        private final String contentHash;
 
-        public BuildScriptRemapper(ClassVisitor cv, ScriptSource source) {
-            super(ASM5, cv);
+        public BuildScriptRemapper(ClassVisitor cv, ScriptSource source, String originalClassName, String contentHash) {
+            super(AsmConstants.ASM_LEVEL, cv);
             this.scriptSource = source;
+            this.originalClassName = originalClassName;
+            this.contentHash = contentHash;
         }
 
         public void visit(int version, int access, String name, String signature, String superName, String[] interfaces) {
-            cv.visit(version, access, remap(name), remap(signature), remap(superName), remap(interfaces));
+            String owner = remap(name);
+            boolean shouldAddScriptOrigin = shouldAddScriptOrigin(access);
+            cv.visit(version, access, owner, remap(signature), remap(superName), remapAndAddInterfaces(interfaces, shouldAddScriptOrigin));
+            if (shouldAddScriptOrigin) {
+                addOriginalClassName(cv, owner, originalClassName);
+                addContentHash(cv, owner, contentHash);
+            }
+        }
+
+        private static boolean shouldAddScriptOrigin(int access) {
+            return ((access & ACC_INTERFACE) == 0) && ((access & ACC_ANNOTATION) == 0);
+        }
+
+        private static void addOriginalClassName(ClassVisitor cv, String owner, String originalClassName) {
+            cv.visitField(ACC_PRIVATE | ACC_STATIC | ACC_SYNTHETIC | ACC_FINAL, "__originalClassName", Type.getDescriptor(String.class), "", originalClassName);
+            MethodVisitor mv = cv.visitMethod(ACC_PUBLIC, "getOriginalClassName", Type.getMethodDescriptor(Type.getType(String.class)), null, null);
+            mv.visitCode();
+            mv.visitFieldInsn(GETSTATIC, owner, "__originalClassName", Type.getDescriptor(String.class));
+            mv.visitInsn(ARETURN);
+            mv.visitMaxs(1, 1);
+            mv.visitEnd();
+        }
+
+        private static void addContentHash(ClassVisitor cv, String owner, String contentHash) {
+            cv.visitField(ACC_PRIVATE | ACC_STATIC | ACC_SYNTHETIC | ACC_FINAL, "__signature", Type.getDescriptor(String.class), "", contentHash);
+            MethodVisitor mv = cv.visitMethod(ACC_PUBLIC, "getContentHash", Type.getMethodDescriptor(Type.getType(String.class)), null, null);
+            mv.visitCode();
+            mv.visitFieldInsn(GETSTATIC, owner, "__signature", Type.getDescriptor(String.class));
+            mv.visitInsn(ARETURN);
+            mv.visitMaxs(1, 1);
+            mv.visitEnd();
         }
 
         @Override
@@ -267,6 +260,21 @@ public class FileCacheBackedScriptClassCompiler implements ScriptClassCompiler,
             cv.visitSource(scriptSource.getFileName(), debug);
         }
 
+        private String[] remapAndAddInterfaces(String[] interfaces, boolean shouldAddScriptOrigin) {
+            if (!shouldAddScriptOrigin) {
+                return remap(interfaces);
+            }
+            if (interfaces == null) {
+                return new String[]{SCRIPT_ORIGIN};
+            }
+            String[] remapped = new String[interfaces.length + 1];
+            for (int i = 0; i < interfaces.length; i++) {
+                remapped[i] = remap(interfaces[i]);
+            }
+            remapped[remapped.length - 1] = SCRIPT_ORIGIN;
+            return remapped;
+        }
+
         private String[] remap(String[] names) {
             if (names == null) {
                 return null;
@@ -282,6 +290,13 @@ public class FileCacheBackedScriptClassCompiler implements ScriptClassCompiler,
             if (name == null) {
                 return null;
             }
+            if (RuleVisitor.SOURCE_URI_TOKEN.equals(name)) {
+                URI uri = scriptSource.getResource().getLocation().getURI();
+                return uri == null ? null : uri.toString();
+            }
+            if (RuleVisitor.SOURCE_DESC_TOKEN.equals(name)) {
+                return scriptSource.getDisplayName();
+            }
             return name.replaceAll(RemappingScriptSource.MAPPED_SCRIPT, scriptSource.getClassName());
         }
 
@@ -289,16 +304,23 @@ public class FileCacheBackedScriptClassCompiler implements ScriptClassCompiler,
             if (o instanceof Type) {
                 return Type.getType(remap(((Type) o).getDescriptor()));
             }
-            if (RuleVisitor.SOURCE_URI_TOKEN.equals(o)) {
-                URI uri = scriptSource.getResource().getLocation().getURI();
-                return uri == null ? null : uri.toString();
-            }
-            if (RuleVisitor.SOURCE_DESC_TOKEN.equals(o)) {
-                return scriptSource.getDisplayName();
+            if (o instanceof String) {
+                return remap((String) o);
             }
             return o;
         }
 
+        private Object[] remap(int count, Object[] original) {
+            if (count == 0) {
+                return EMPTY_OBJECT_ARRAY;
+            }
+            Object[] remapped = new Object[count];
+            for (int idx = 0; idx < count; idx++) {
+                remapped[idx] = remap(original[idx]);
+            }
+            return remapped;
+        }
+
         public MethodVisitor visitMethod(int access, String name, String desc, String signature, String[] exceptions) {
             MethodVisitor mv = cv.visitMethod(access, name, remap(desc), remap(signature), remap(exceptions));
             if (mv != null && (access & ACC_ABSTRACT) == 0) {
@@ -330,7 +352,7 @@ public class FileCacheBackedScriptClassCompiler implements ScriptClassCompiler,
         class MethodRenamer extends MethodVisitor {
 
             public MethodRenamer(final MethodVisitor mv) {
-                super(ASM5, mv);
+                super(AsmConstants.ASM_LEVEL, mv);
             }
 
             public void visitTypeInsn(int i, String name) {
@@ -360,8 +382,11 @@ public class FileCacheBackedScriptClassCompiler implements ScriptClassCompiler,
                 return super.visitAnnotation(remap(desc), visible);
             }
 
+            @Override
+            public void visitFrame(int type, int nLocal, Object[] local, int nStack, Object[] stack) {
+                super.visitFrame(type, nLocal, remap(nLocal, local), nStack, remap(nStack, stack));
+            }
         }
-
     }
 
     private class RemapBuildScriptsAction<M, T extends Script> implements Action<PersistentCache> {
@@ -388,20 +413,22 @@ public class FileCacheBackedScriptClassCompiler implements ScriptClassCompiler,
         }
 
         public void execute(final PersistentCache remappedClassesCache) {
-            final PersistentCache cache = cacheRepository.cache(String.format("scripts/%s/%s/%s", sourceHash, dslId, classpathHash))
-                .withValidator(validator)
-                .withDisplayName(String.format("%s generic class cache for %s", dslId, source.getDisplayName()))
+            final PersistentCache cache = cacheRepository.cache("scripts/" + sourceHash + "/" + dslId + "/" + classpathHash)
+                .withDisplayName(dslId + " generic class cache for " + source.getDisplayName())
                 .withInitializer(new ProgressReportingInitializer(
                     progressLoggerFactory,
                     new CompileToCrossBuildCacheAction(remapped, classLoader, operation, verifier, scriptBaseClass),
                     "Compiling script into cache",
                     "Compiling " + source.getDisplayName() + " to cross build script cache"))
                 .open();
-            cache.close();
-            final File genericClassesDir = classesDir(cache);
-            final File metadataDir = metadataDir(cache);
-            remapClasses(genericClassesDir, classesDir(remappedClassesCache), remapped);
-            copyMetadata(metadataDir, metadataDir(remappedClassesCache));
+            try {
+                final File genericClassesDir = classesDir(cache);
+                final File metadataDir = metadataDir(cache);
+                remapClasses(genericClassesDir, classesDir(remappedClassesCache), remapped);
+                copyMetadata(metadataDir, metadataDir(remappedClassesCache));
+            } finally {
+                cache.close();
+            }
         }
 
         private void remapClasses(File scriptCacheDir, File relocalizedDir, RemappingScriptSource source) {
@@ -418,9 +445,12 @@ public class FileCacheBackedScriptClassCompiler implements ScriptClassCompiler,
                         renamed = className + renamed.substring(RemappingScriptSource.MAPPED_SCRIPT.length());
                     }
                     ClassWriter cv = new ClassWriter(0);
-                    BuildScriptRemapper remapper = new BuildScriptRemapper(cv, origin);
                     try {
-                        ClassReader cr = new ClassReader(Files.toByteArray(file));
+                        byte[] contents = Files.toByteArray(file);
+                        ClassReader cr = new ClassReader(contents);
+                        String originalClassName = cr.getClassName();
+                        String contentHash = Hashing.hashBytes(contents).toString();
+                        BuildScriptRemapper remapper = new BuildScriptRemapper(cv, origin, originalClassName, contentHash);
                         cr.accept(remapper, 0);
                         Files.write(cv.toByteArray(), new File(relocalizedDir, renamed));
                     } catch (IOException ex) {
