@@ -23,56 +23,69 @@ import org.gradle.api.ProjectState;
 import org.gradle.api.Task;
 import org.gradle.api.artifacts.DependencyResolutionListener;
 import org.gradle.api.artifacts.ResolvableDependencies;
+import org.gradle.api.artifacts.component.ComponentArtifactIdentifier;
 import org.gradle.api.execution.TaskExecutionListener;
 import org.gradle.api.initialization.Settings;
+import org.gradle.api.internal.artifacts.transform.ArtifactTransformListener;
+import org.gradle.api.internal.artifacts.transform.ArtifactTransformer;
 import org.gradle.api.invocation.Gradle;
 import org.gradle.api.tasks.TaskState;
 import org.gradle.initialization.BuildCompletionListener;
-import org.gradle.initialization.BuildRequestMetaData;
-import org.gradle.internal.TimeProvider;
+import org.gradle.internal.buildevents.BuildStartedTime;
+import org.gradle.internal.time.Clock;
+
+import javax.annotation.Nullable;
+import java.io.File;
 
 /**
  * Adapts various events to build a {@link BuildProfile} model, and then notifies a {@link ReportGeneratingProfileListener} when the model is ready.
  */
-public class ProfileEventAdapter implements BuildListener, ProjectEvaluationListener, TaskExecutionListener, DependencyResolutionListener, BuildCompletionListener {
-    private final BuildRequestMetaData buildMetaData;
-    private final TimeProvider timeProvider;
+public class ProfileEventAdapter implements BuildListener, ProjectEvaluationListener, TaskExecutionListener, DependencyResolutionListener, BuildCompletionListener, ArtifactTransformListener {
+    private final BuildStartedTime buildStartedTime;
+    private final Clock clock;
     private final ProfileListener listener;
+    private final ThreadLocal<ContinuousOperation> currentTransform = new ThreadLocal<ContinuousOperation>();
     private BuildProfile buildProfile;
 
-    public ProfileEventAdapter(BuildRequestMetaData buildMetaData, TimeProvider timeProvider, ProfileListener listener) {
-        this.buildMetaData = buildMetaData;
-        this.timeProvider = timeProvider;
+    public ProfileEventAdapter(BuildStartedTime buildStartedTime, Clock clock, ProfileListener listener) {
+        this.buildStartedTime = buildStartedTime;
+        this.clock = clock;
         this.listener = listener;
     }
 
     // BuildListener
+    @Override
     public void buildStarted(Gradle gradle) {
-        long now = timeProvider.getCurrentTime();
+        long now = clock.getCurrentTime();
         buildProfile = new BuildProfile(gradle.getStartParameter());
         buildProfile.setBuildStarted(now);
-        buildProfile.setProfilingStarted(buildMetaData.getBuildTimeClock().getStartTime());
+        buildProfile.setProfilingStarted(buildStartedTime.getStartTime());
     }
 
+    @Override
     public void settingsEvaluated(Settings settings) {
-        buildProfile.setSettingsEvaluated(timeProvider.getCurrentTime());
+        buildProfile.setSettingsEvaluated(clock.getCurrentTime());
     }
 
+    @Override
     public void projectsLoaded(Gradle gradle) {
-        buildProfile.setProjectsLoaded(timeProvider.getCurrentTime());
+        buildProfile.setProjectsLoaded(clock.getCurrentTime());
     }
 
+    @Override
     public void projectsEvaluated(Gradle gradle) {
-        buildProfile.setProjectsEvaluated(timeProvider.getCurrentTime());
+        buildProfile.setProjectsEvaluated(clock.getCurrentTime());
     }
 
+    @Override
     public void buildFinished(BuildResult result) {
         buildProfile.setSuccessful(result.getFailure() == null);
     }
 
+    @Override
     public void completed() {
-        if(buildProfile != null) {
-            buildProfile.setBuildFinished(timeProvider.getCurrentTime());
+        if (buildProfile != null) {
+            buildProfile.setBuildFinished(clock.getCurrentTime());
             try {
                 listener.buildFinished(buildProfile);
             } finally {
@@ -82,27 +95,31 @@ public class ProfileEventAdapter implements BuildListener, ProjectEvaluationList
     }
 
     // ProjectEvaluationListener
+    @Override
     public void beforeEvaluate(Project project) {
-        long now = timeProvider.getCurrentTime();
+        long now = clock.getCurrentTime();
         buildProfile.getProjectProfile(project.getPath()).getConfigurationOperation().setStart(now);
     }
 
+    @Override
     public void afterEvaluate(Project project, ProjectState state) {
-        long now = timeProvider.getCurrentTime();
+        long now = clock.getCurrentTime();
         ProjectProfile projectProfile = buildProfile.getProjectProfile(project.getPath());
         projectProfile.getConfigurationOperation().setFinish(now);
     }
 
     // TaskExecutionListener
+    @Override
     public void beforeExecute(Task task) {
-        long now = timeProvider.getCurrentTime();
+        long now = clock.getCurrentTime();
         Project project = task.getProject();
         ProjectProfile projectProfile = buildProfile.getProjectProfile(project.getPath());
         projectProfile.getTaskProfile(task.getPath()).setStart(now);
     }
 
+    @Override
     public void afterExecute(Task task, TaskState state) {
-        long now = timeProvider.getCurrentTime();
+        long now = clock.getCurrentTime();
         Project project = task.getProject();
         ProjectProfile projectProfile = buildProfile.getProjectProfile(project.getPath());
         TaskExecution taskExecution = projectProfile.getTaskProfile(task.getPath());
@@ -111,14 +128,37 @@ public class ProfileEventAdapter implements BuildListener, ProjectEvaluationList
     }
 
     // DependencyResolutionListener
+    @Override
     public void beforeResolve(ResolvableDependencies dependencies) {
-        long now = timeProvider.getCurrentTime();
+        long now = clock.getCurrentTime();
         buildProfile.getDependencySetProfile(dependencies.getPath()).setStart(now);
     }
 
+    @Override
     public void afterResolve(ResolvableDependencies dependencies) {
-        long now = timeProvider.getCurrentTime();
+        long now = clock.getCurrentTime();
         buildProfile.getDependencySetProfile(dependencies.getPath()).setFinish(now);
     }
-}
 
+    @Override
+    public void beforeTransform(ArtifactTransformer transform, @Nullable ComponentArtifactIdentifier artifactId, File file) {
+        long now = clock.getCurrentTime();
+        StringBuilder transformDescription = new StringBuilder(256);
+        if (artifactId != null) {
+            transformDescription.append(artifactId.getDisplayName());
+        } else {
+            transformDescription.append(file.getAbsolutePath());
+        }
+        transformDescription.append(" with ");
+        transformDescription.append(transform.getDisplayName());
+        FragmentedOperation transformProfile = buildProfile.getTransformProfile(transformDescription.toString());
+        currentTransform.set(transformProfile.start(now));
+    }
+
+    @Override
+    public void afterTransform(ArtifactTransformer transform, @Nullable ComponentArtifactIdentifier artifactId, File file, @Nullable Throwable failure) {
+        long now = clock.getCurrentTime();
+        currentTransform.get().setFinish(now);
+        currentTransform.remove();
+    }
+}
