@@ -15,48 +15,39 @@
  */
 package org.gradle.tooling.internal.consumer;
 
-import com.google.common.base.Preconditions;
-import org.gradle.api.internal.classpath.EffectiveClassPath;
+import org.gradle.api.internal.classpath.DefaultModuleRegistry;
+import org.gradle.api.internal.classpath.Module;
 import org.gradle.initialization.BuildCancellationToken;
 import org.gradle.initialization.layout.BuildLayout;
 import org.gradle.initialization.layout.BuildLayoutFactory;
-import org.gradle.internal.Factory;
-import org.gradle.internal.UncheckedException;
 import org.gradle.internal.classpath.ClassPath;
 import org.gradle.internal.classpath.DefaultClassPath;
-import org.gradle.internal.logging.progress.ProgressLogger;
 import org.gradle.internal.logging.progress.ProgressLoggerFactory;
+import org.gradle.internal.time.Clock;
 import org.gradle.tooling.BuildCancelledException;
 import org.gradle.tooling.GradleConnectionException;
+import org.gradle.tooling.internal.protocol.InternalBuildProgressListener;
 import org.gradle.util.DistributionLocator;
 import org.gradle.util.GradleVersion;
-import org.gradle.wrapper.Download;
 import org.gradle.wrapper.GradleUserHomeLookup;
-import org.gradle.wrapper.IDownload;
-import org.gradle.wrapper.Install;
-import org.gradle.wrapper.Logger;
-import org.gradle.wrapper.PathAssembler;
 import org.gradle.wrapper.WrapperConfiguration;
 import org.gradle.wrapper.WrapperExecutor;
 
 import java.io.File;
+import java.io.FileFilter;
 import java.io.FileNotFoundException;
 import java.net.URI;
-import java.util.LinkedHashSet;
-import java.util.concurrent.Callable;
+import java.util.Arrays;
 import java.util.concurrent.CancellationException;
-import java.util.concurrent.ExecutionException;
-import java.util.concurrent.ExecutorService;
-import java.util.concurrent.Future;
 
 import static org.gradle.internal.FileUtils.hasExtension;
 
 public class DistributionFactory {
-    private final Factory<? extends ExecutorService> executorFactory;
+    private final Clock clock;
     private File distributionBaseDir;
 
-    public DistributionFactory(Factory<? extends ExecutorService> executorFactory) {
-        this.executorFactory = Preconditions.checkNotNull(executorFactory);
+    public DistributionFactory(Clock clock) {
+        this.clock = clock;
     }
 
     public void setDistributionBaseDir(File distributionBaseDir) {
@@ -70,7 +61,7 @@ public class DistributionFactory {
         BuildLayout layout = new BuildLayoutFactory().getLayoutFor(projectDir, searchUpwards);
         WrapperExecutor wrapper = WrapperExecutor.forProjectDirectory(layout.getRootDirectory());
         if (wrapper.getDistribution() != null) {
-            return new ZippedDistribution(wrapper.getConfiguration(), executorFactory, distributionBaseDir);
+            return new ZippedDistribution(wrapper.getConfiguration(), distributionBaseDir, clock);
         }
         return getDownloadedDistribution(GradleVersion.current().getVersion());
     }
@@ -80,7 +71,7 @@ public class DistributionFactory {
      */
     public Distribution getDistribution(File gradleHomeDir) {
         return new InstalledDistribution(gradleHomeDir, "Gradle installation '" + gradleHomeDir + "'",
-                "Gradle installation directory '" + gradleHomeDir + "'");
+            "Gradle installation directory '" + gradleHomeDir + "'");
     }
 
     /**
@@ -96,7 +87,7 @@ public class DistributionFactory {
     public Distribution getDistribution(URI gradleDistribution) {
         WrapperConfiguration configuration = new WrapperConfiguration();
         configuration.setDistribution(gradleDistribution);
-        return new ZippedDistribution(configuration, executorFactory, distributionBaseDir);
+        return new ZippedDistribution(configuration, distributionBaseDir, clock);
     }
 
     /**
@@ -114,71 +105,44 @@ public class DistributionFactory {
     private static class ZippedDistribution implements Distribution {
         private InstalledDistribution installedDistribution;
         private final WrapperConfiguration wrapperConfiguration;
-        private final Factory<? extends ExecutorService> executorFactory;
         private final File distributionBaseDir;
+        private final Clock clock;
 
-        private ZippedDistribution(WrapperConfiguration wrapperConfiguration, Factory<? extends ExecutorService> executorFactory, File distributionBaseDir) {
+        private ZippedDistribution(WrapperConfiguration wrapperConfiguration, File distributionBaseDir, Clock clock) {
             this.wrapperConfiguration = wrapperConfiguration;
-            this.executorFactory = executorFactory;
             this.distributionBaseDir = distributionBaseDir;
+            this.clock = clock;
         }
 
         public String getDisplayName() {
             return "Gradle distribution '" + wrapperConfiguration.getDistribution() + "'";
         }
 
-        public ClassPath getToolingImplementationClasspath(final ProgressLoggerFactory progressLoggerFactory, final File userHomeDir, BuildCancellationToken cancellationToken) {
+        public ClassPath getToolingImplementationClasspath(ProgressLoggerFactory progressLoggerFactory, final InternalBuildProgressListener progressListener, final File userHomeDir, BuildCancellationToken cancellationToken) {
             if (installedDistribution == null) {
-                Callable<File> installDistroTask = new Callable<File>() {
-                    public File call() throws Exception {
-                        File installDir;
-                        try {
-                            File realUserHomeDir = determineRealUserHomeDir(userHomeDir);
-                            Install install = new Install(new Logger(false), new ProgressReportingDownload(progressLoggerFactory), new PathAssembler(realUserHomeDir));
-                            installDir = install.createDist(wrapperConfiguration);
-                        } catch (FileNotFoundException e) {
-                            throw new IllegalArgumentException(String.format("The specified %s does not exist.", getDisplayName()), e);
-                        } catch (CancellationException e) {
-                            throw new BuildCancelledException(String.format("Distribution download cancelled. Using distribution from '%s'.", wrapperConfiguration.getDistribution()), e);
-                        } catch (Exception e) {
-                            throw new GradleConnectionException(String.format("Could not install Gradle distribution from '%s'.", wrapperConfiguration.getDistribution()), e);
-                        }
-                        return installDir;
-                    }
-                };
+                final DistributionInstaller installer = new DistributionInstaller(progressLoggerFactory, progressListener, clock);
                 File installDir;
-                ExecutorService executor = null;
                 try {
-                    executor = executorFactory.create();
-                    final Future<File> installDirFuture = executor.submit(installDistroTask);
                     cancellationToken.addCallback(new Runnable() {
                         public void run() {
-                            // TODO(radim): better to close the connection too to allow quick finish of the task
-                            installDirFuture.cancel(true);
+                            installer.cancel();
                         }
                     });
-                    installDir = installDirFuture.get();
+                    installDir = installer.install(determineRealUserHomeDir(userHomeDir), wrapperConfiguration);
                 } catch (CancellationException e) {
                     throw new BuildCancelledException(String.format("Distribution download cancelled. Using distribution from '%s'.", wrapperConfiguration.getDistribution()), e);
-                } catch (InterruptedException e) {
+                } catch (FileNotFoundException e) {
+                    throw new IllegalArgumentException(String.format("The specified %s does not exist.", getDisplayName()), e);
+                } catch (Exception e) {
                     throw new GradleConnectionException(String.format("Could not install Gradle distribution from '%s'.", wrapperConfiguration.getDistribution()), e);
-                } catch (ExecutionException e) {
-                    if (e.getCause() != null) {
-                        UncheckedException.throwAsUncheckedException(e.getCause());
-                    }
-                    throw new GradleConnectionException(String.format("Could not install Gradle distribution from '%s'.", wrapperConfiguration.getDistribution()), e);
-                } finally {
-                    if (executor != null) {
-                        executor.shutdown();
-                    }
                 }
                 installedDistribution = new InstalledDistribution(installDir, getDisplayName(), getDisplayName());
             }
-            return installedDistribution.getToolingImplementationClasspath(progressLoggerFactory, userHomeDir, cancellationToken);
+            return installedDistribution.getToolingImplementationClasspath(progressLoggerFactory, progressListener, userHomeDir, cancellationToken);
         }
 
         private File determineRealUserHomeDir(final File userHomeDir) {
-            if(distributionBaseDir != null) {
+            if (distributionBaseDir != null) {
                 return distributionBaseDir;
             }
 
@@ -186,31 +150,12 @@ public class DistributionFactory {
         }
     }
 
-    private static class ProgressReportingDownload implements IDownload {
-        private final ProgressLoggerFactory progressLoggerFactory;
-
-        private ProgressReportingDownload(ProgressLoggerFactory progressLoggerFactory) {
-            this.progressLoggerFactory = progressLoggerFactory;
-        }
-
-        public void download(URI address, File destination) throws Exception {
-            ProgressLogger progressLogger = progressLoggerFactory.newOperation(DistributionFactory.class);
-            progressLogger.setDescription("Download " + address);
-            progressLogger.started();
-            try {
-                new Download(new Logger(false), "Gradle Tooling API", GradleVersion.current().getVersion()).download(address, destination);
-            } finally {
-                progressLogger.completed();
-            }
-        }
-    }
-
     private static class InstalledDistribution implements Distribution {
         private final File gradleHomeDir;
         private final String displayName;
         private final String locationDisplayName;
 
-        public InstalledDistribution(File gradleHomeDir, String displayName, String locationDisplayName) {
+        InstalledDistribution(File gradleHomeDir, String displayName, String locationDisplayName) {
             this.gradleHomeDir = gradleHomeDir;
             this.displayName = displayName;
             this.locationDisplayName = locationDisplayName;
@@ -220,18 +165,7 @@ public class DistributionFactory {
             return displayName;
         }
 
-        public ClassPath getToolingImplementationClasspath(ProgressLoggerFactory progressLoggerFactory, File userHomeDir, BuildCancellationToken cancellationToken) {
-            ProgressLogger progressLogger = progressLoggerFactory.newOperation(DistributionFactory.class);
-            progressLogger.setDescription("Validate distribution");
-            progressLogger.started();
-            try {
-                return getToolingImpl();
-            } finally {
-                progressLogger.completed();
-            }
-        }
-
-        private ClassPath getToolingImpl() {
+        public ClassPath getToolingImplementationClasspath(ProgressLoggerFactory progressLoggerFactory, InternalBuildProgressListener progressListener, File userHomeDir, BuildCancellationToken cancellationToken) {
             if (!gradleHomeDir.exists()) {
                 throw new IllegalArgumentException(String.format("The specified %s does not exist.", locationDisplayName));
             }
@@ -242,13 +176,15 @@ public class DistributionFactory {
             if (!libDir.isDirectory()) {
                 throw new IllegalArgumentException(String.format("The specified %s does not appear to contain a Gradle distribution.", locationDisplayName));
             }
-            LinkedHashSet<File> files = new LinkedHashSet<File>();
-            for (File file : libDir.listFiles()) {
-                if (hasExtension(file, ".jar")) {
-                    files.add(file);
+            File[] files = libDir.listFiles(new FileFilter() {
+                @Override
+                public boolean accept(File file) {
+                    return hasExtension(file, ".jar");
                 }
-            }
-            return new DefaultClassPath(files);
+            });
+            // Make sure file order is always consistent
+            Arrays.sort(files);
+            return DefaultClassPath.of(files);
         }
     }
 
@@ -257,8 +193,13 @@ public class DistributionFactory {
             return "Gradle classpath distribution";
         }
 
-        public ClassPath getToolingImplementationClasspath(ProgressLoggerFactory progressLoggerFactory, File userHomeDir, BuildCancellationToken cancellationToken) {
-            return new EffectiveClassPath(getClass().getClassLoader());
+        public ClassPath getToolingImplementationClasspath(ProgressLoggerFactory progressLoggerFactory, InternalBuildProgressListener progressListener, File userHomeDir, BuildCancellationToken cancellationToken) {
+            DefaultModuleRegistry registry = new DefaultModuleRegistry(null);
+            ClassPath classpath = ClassPath.EMPTY;
+            for (Module module : registry.getModule("gradle-launcher").getAllRequiredModules()) {
+                classpath = classpath.plus(module.getClasspath());
+            }
+            return classpath;
         }
     }
 }
