@@ -25,6 +25,9 @@ import org.codehaus.groovy.runtime.InvokerHelper;
 
 import java.io.File;
 import java.io.IOException;
+import java.lang.reflect.Method;
+
+import static org.codehaus.groovy.runtime.MetaClassHelper.capitalize;
 
 /**
  * This object represents a Groovy script
@@ -57,12 +60,29 @@ public abstract class Script extends GroovyObjectSupport {
     }
 
     public void setProperty(String property, Object newValue) {
-        if ("binding".equals(property))
+        if ("binding".equals(property)) {
             setBinding((Binding) newValue);
-        else if("metaClass".equals(property))
-            setMetaClass((MetaClass)newValue);
-        else
+        } else if ("metaClass".equals(property)) {
+            setMetaClass((MetaClass) newValue);
+        } else if (!binding.hasVariable(property)
+                // GROOVY-9554: @Field adds setter
+                && hasSetterMethodFor(property)) {
+            super.setProperty(property, newValue);
+        } else {
             binding.setVariable(property, newValue);
+        }
+    }
+
+    private boolean hasSetterMethodFor(String property) {
+        String setterName = "set" + capitalize(property);
+        for (Method method : getClass().getDeclaredMethods()) {
+            if (method.getParameterTypes().length == 1
+                    // TODO: Test modifiers or return type?
+                    && method.getName().equals(setterName)) {
+                return true;
+            }
+        }
+        return false;
     }
 
     /**
