@@ -1,5 +1,5 @@
 /*
- * Copyright 2017-2019 original authors
+ * Copyright 2017-2020 original authors
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -20,6 +20,7 @@ import io.micronaut.inject.BeanDefinition;
 import io.micronaut.inject.configuration.ConfigurationMetadataBuilder;
 import org.objectweb.asm.Type;
 
+import javax.annotation.Nonnull;
 import javax.annotation.Nullable;
 import java.io.File;
 import java.io.IOException;
@@ -67,6 +68,12 @@ public interface BeanDefinitionVisitor {
                                         Map<String, AnnotationMetadata> argumentAnnotationMetadata,
                                         Map<String, Map<String, Object>> genericTypes);
 
+    /**
+     * @return The name of the bean definition reference class.
+     */
+    @Nonnull
+    String getBeanDefinitionReferenceClassName();
+
     /**
      * @return Whether the provided type an interface
      */
@@ -287,6 +294,8 @@ public interface BeanDefinitionVisitor {
      * @param argumentAnnotationMetadata The argument annotation metadata
      * @param genericTypes               The generic types of each argument. Can be null.
      * @param annotationMetadata         The annotation metadata for the method
+     * @param isInterface                If the method belongs to an interface
+     * @param isDefault                  If the method is a default method
      * @return The {@link ExecutableMethodWriter}.
      */
     ExecutableMethodWriter visitExecutableMethod(Object declaringType,
@@ -298,7 +307,9 @@ public interface BeanDefinitionVisitor {
                                                  Map<String, Object> genericArgumentTypes,
                                                  Map<String, AnnotationMetadata> argumentAnnotationMetadata,
                                                  Map<String, Map<String, Object>> genericTypes,
-                                                 @Nullable AnnotationMetadata annotationMetadata);
+                                                 @Nullable AnnotationMetadata annotationMetadata,
+                                                 boolean isInterface,
+                                                 boolean isDefault);
 
     /**
      * Visits a field injection point.
@@ -358,13 +369,15 @@ public interface BeanDefinitionVisitor {
      * @param field              The name of the field that represents the builder
      * @param annotationMetadata The annotation metadata associated with the field
      * @param metadataBuilder    The {@link ConfigurationMetadataBuilder}
+     * @param isInterface        Whether the builder type is an interface or not
      * @see io.micronaut.context.annotation.ConfigurationBuilder
      */
     void visitConfigBuilderField(
             Object type,
             String field,
             AnnotationMetadata annotationMetadata,
-            ConfigurationMetadataBuilder metadataBuilder);
+            ConfigurationMetadataBuilder metadataBuilder,
+            boolean isInterface);
 
     /**
      * Begin defining a configuration builder.
@@ -373,13 +386,15 @@ public interface BeanDefinitionVisitor {
      * @param methodName         The name of the method that returns the builder
      * @param annotationMetadata The annotation metadata associated with the field
      * @param metadataBuilder    The {@link ConfigurationMetadataBuilder}
+     * @param isInterface        Whether the builder type is an interface or not
      * @see io.micronaut.context.annotation.ConfigurationBuilder
      */
     void visitConfigBuilderMethod(
             Object type,
             String methodName,
             AnnotationMetadata annotationMetadata,
-            ConfigurationMetadataBuilder metadataBuilder);
+            ConfigurationMetadataBuilder metadataBuilder,
+            boolean isInterface);
 
     /**
      * Visit a configuration builder method.
