@@ -1,5 +1,5 @@
 /*
- * Copyright 2017-2019 original authors
+ * Copyright 2017-2020 original authors
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -22,7 +22,6 @@ import io.micronaut.core.reflect.ReflectionUtils;
 import io.micronaut.core.type.Argument;
 import io.micronaut.inject.ExecutableMethod;
 import io.micronaut.inject.annotation.AnnotationMetadataReference;
-import io.micronaut.inject.annotation.AnnotationMetadataWriter;
 import io.micronaut.inject.annotation.DefaultAnnotationMetadata;
 import org.objectweb.asm.ClassWriter;
 import org.objectweb.asm.MethodVisitor;
@@ -54,6 +53,8 @@ public class ExecutableMethodWriter extends AbstractAnnotationMetadataWriter imp
         ReflectionUtils.getRequiredInternalMethod(AbstractExecutableMethod.class, "invokeInternal", Object.class, Object[].class));
     protected static final org.objectweb.asm.commons.Method METHOD_IS_ABSTRACT = org.objectweb.asm.commons.Method.getMethod(
             ReflectionUtils.getRequiredInternalMethod(ExecutableMethod.class, "isAbstract"));
+    protected static final org.objectweb.asm.commons.Method METHOD_IS_SUSPEND = org.objectweb.asm.commons.Method.getMethod(
+            ReflectionUtils.getRequiredInternalMethod(ExecutableMethod.class, "isSuspend"));
     protected static final Method METHOD_GET_TARGET = Method.getMethod("java.lang.reflect.Method resolveTargetMethod()");
     private  static final Type TYPE_REFLECTION_UTILS = Type.getType(ReflectionUtils.class);
     private static final org.objectweb.asm.commons.Method METHOD_GET_REQUIRED_METHOD = org.objectweb.asm.commons.Method.getMethod(
@@ -68,15 +69,18 @@ public class ExecutableMethodWriter extends AbstractAnnotationMetadataWriter imp
     private final String methodProxyShortName;
     private final boolean isInterface;
     private final boolean isAbstract;
+    private final boolean isSuspend;
+    private final boolean isDefault;
     private String outerClassName = null;
     private boolean isStatic = false;
-    private final Map<String, GeneratorAdapter> loadTypeMethods = new HashMap<>();
 
     /**
      * @param beanFullClassName    The bean full class name
      * @param methodClassName      The method class name
      * @param methodProxyShortName The method proxy short name
      * @param isInterface          Whether is an interface
+     * @param isDefault            Whether the method is a default method
+     * @param isSuspend            Whether the method is Kotlin suspend function
      * @param annotationMetadata   The annotation metadata
      */
     public ExecutableMethodWriter(
@@ -84,6 +88,8 @@ public class ExecutableMethodWriter extends AbstractAnnotationMetadataWriter imp
             String methodClassName,
             String methodProxyShortName,
             boolean isInterface,
+            boolean isDefault,
+            boolean isSuspend,
             AnnotationMetadata annotationMetadata) {
         super(methodClassName, annotationMetadata, true);
         this.classWriter = new ClassWriter(ClassWriter.COMPUTE_MAXS | ClassWriter.COMPUTE_FRAMES);
@@ -93,7 +99,9 @@ public class ExecutableMethodWriter extends AbstractAnnotationMetadataWriter imp
         this.internalName = getInternalName(methodClassName);
         this.methodType = getObjectType(methodClassName);
         this.isInterface = isInterface;
-        this.isAbstract = isInterface;
+        this.isAbstract = !isInterface || !isDefault;
+        this.isDefault = isDefault;
+        this.isSuspend = isSuspend;
     }
 
 
@@ -103,6 +111,8 @@ public class ExecutableMethodWriter extends AbstractAnnotationMetadataWriter imp
      * @param methodProxyShortName The method proxy short name
      * @param isInterface          Whether is an interface
      * @param isAbstract           Whether the method is abstract
+     * @param isDefault            Whether the method is a default method
+     * @param isSuspend            Whether the method is Kotlin suspend function
      * @param annotationMetadata   The annotation metadata
      */
     public ExecutableMethodWriter(
@@ -111,6 +121,8 @@ public class ExecutableMethodWriter extends AbstractAnnotationMetadataWriter imp
             String methodProxyShortName,
             boolean isInterface,
             boolean isAbstract,
+            boolean isDefault,
+            boolean isSuspend,
             AnnotationMetadata annotationMetadata) {
         super(methodClassName, annotationMetadata, true);
         this.classWriter = new ClassWriter(ClassWriter.COMPUTE_MAXS | ClassWriter.COMPUTE_FRAMES);
@@ -120,7 +132,9 @@ public class ExecutableMethodWriter extends AbstractAnnotationMetadataWriter imp
         this.internalName = getInternalName(methodClassName);
         this.methodType = getObjectType(methodClassName);
         this.isInterface = isInterface;
-        this.isAbstract = isInterface || isAbstract;
+        this.isAbstract = isAbstract;
+        this.isDefault = isDefault;
+        this.isSuspend = isSuspend;
     }
 
     /**
@@ -130,6 +144,28 @@ public class ExecutableMethodWriter extends AbstractAnnotationMetadataWriter imp
         return isAbstract;
     }
 
+    /**
+     * @return Is the method in an interface.
+     */
+    public boolean isInterface() {
+        return isInterface;
+    }
+
+
+    /**
+     * @return Is the method a default method.
+     */
+    public boolean isDefault() {
+        return isDefault;
+    }
+
+    /**
+     * @return Is the method suspend.
+     */
+    public boolean isSuspend() {
+        return isSuspend;
+    }
+
     /**
      * @return The class name
      */
@@ -299,6 +335,23 @@ public class ExecutableMethodWriter extends AbstractAnnotationMetadataWriter imp
         isAbstractMethod.visitMaxs(1, 1);
         isAbstractMethod.endMethod();
 
+        // add isSuspend method
+        GeneratorAdapter isSuspendMethod = new GeneratorAdapter(classWriter.visitMethod(
+                ACC_PUBLIC | ACC_FINAL,
+                METHOD_IS_SUSPEND.getName(),
+                METHOD_IS_SUSPEND.getDescriptor(),
+                null,
+                null),
+                ACC_PUBLIC,
+                METHOD_IS_SUSPEND.getName(),
+                METHOD_IS_SUSPEND.getDescriptor()
+        );
+
+        isSuspendMethod.push(isSuspend());
+        isSuspendMethod.returnValue();
+        isSuspendMethod.visitMaxs(1, 1);
+        isSuspendMethod.endMethod();
+
         // invoke the methods with the passed arguments
         String invokeDescriptor = METHOD_INVOKE_INTERNAL.getDescriptor();
         String invokeInternalName = METHOD_INVOKE_INTERNAL.getName();
@@ -334,10 +387,6 @@ public class ExecutableMethodWriter extends AbstractAnnotationMetadataWriter imp
 
     @Override
     public void accept(ClassWriterOutputVisitor classWriterOutputVisitor) throws IOException {
-        AnnotationMetadataWriter annotationMetadataWriter = getAnnotationMetadataWriter();
-        if (annotationMetadataWriter != null) {
-            annotationMetadataWriter.accept(classWriterOutputVisitor);
-        }
         try (OutputStream outputStream = classWriterOutputVisitor.visitClass(className)) {
             outputStream.write(classWriter.toByteArray());
         }
