@@ -1,5 +1,5 @@
 /*
- * Copyright 2017-2018 original authors
+ * Copyright 2017-2020 original authors
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -13,7 +13,6 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-
 package io.micronaut.spring.tx.annotation;
 
 import io.micronaut.aop.MethodInterceptor;
@@ -30,8 +29,8 @@ import org.springframework.transaction.PlatformTransactionManager;
 import org.springframework.transaction.TransactionSystemException;
 import org.springframework.transaction.annotation.Isolation;
 import org.springframework.transaction.annotation.Propagation;
+import org.springframework.transaction.interceptor.TransactionAspectSupport;
 import org.springframework.transaction.interceptor.TransactionAttribute;
-import org.springframework.transaction.support.TransactionTemplate;
 
 import javax.inject.Singleton;
 import java.util.Map;
@@ -44,7 +43,7 @@ import java.util.concurrent.ConcurrentHashMap;
  * @since 1.0
  */
 @Singleton
-public class TransactionInterceptor implements MethodInterceptor<Object, Object> {
+public class TransactionInterceptor extends TransactionAspectSupport implements MethodInterceptor<Object, Object> {
 
     private final Map<ExecutableMethod, TransactionAttribute> transactionDefinitionMap = new ConcurrentHashMap<>();
     private final Map<String, PlatformTransactionManager> transactionManagerMap = new ConcurrentHashMap<>();
@@ -59,8 +58,8 @@ public class TransactionInterceptor implements MethodInterceptor<Object, Object>
 
     @Override
     public final Object intercept(MethodInvocationContext<Object, Object> context) {
-        if (context.hasAnnotation(Transactional.class)) {
-            String transactionManagerName = context.getValue(Transactional.class, String.class).orElse(null);
+        if (context.hasStereotype(Transactional.class)) {
+            String transactionManagerName = context.findAnnotation(Transactional.class).flatMap(AnnotationValue::stringValue).orElse(null);
             if (StringUtils.isEmpty(transactionManagerName)) {
                 transactionManagerName = null;
             }
@@ -73,17 +72,31 @@ public class TransactionInterceptor implements MethodInterceptor<Object, Object>
                 finalTransactionManagerName
             );
 
-            TransactionTemplate template = new TransactionTemplate(
-                transactionManager,
-                transactionDefinition
-            );
-
-            return template.execute(status -> context.proceed());
+            final TransactionInfo transactionInfo = createTransactionIfNecessary(
+                    transactionManager,
+                    transactionDefinition,
+                    context.getDeclaringType().getName() + "." + context.getMethodName());
+            Object retVal;
+            try {
+                retVal = context.proceed();
+            } catch (Throwable ex) {
+                completeTransactionAfterThrowing(transactionInfo, ex);
+                throw ex;
+            } finally {
+                cleanupTransactionInfo(transactionInfo);
+            }
+            commitTransactionAfterReturning(transactionInfo);
+            return retVal;
         } else {
             return context.proceed();
         }
     }
 
+    @Override
+    public void afterPropertiesSet() {
+        // no-op - override this because TransactionAspectSupport provides some undesired validation behaviour
+    }
+
     /**
      * @param targetMethod           The target method
      * @param annotationMetadata     The annotation metadata
