@@ -1,11 +1,11 @@
 /*
- * Copyright 2012-2018 the original author or authors.
+ * Copyright 2012-2019 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
  *
- *      http://www.apache.org/licenses/LICENSE-2.0
+ *      https://www.apache.org/licenses/LICENSE-2.0
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
@@ -18,6 +18,7 @@ package org.springframework.boot.loader.tools;
 
 import java.io.BufferedInputStream;
 import java.io.ByteArrayInputStream;
+import java.io.ByteArrayOutputStream;
 import java.io.File;
 import java.io.FileInputStream;
 import java.io.FileNotFoundException;
@@ -37,29 +38,33 @@ import java.util.Set;
 import java.util.jar.JarEntry;
 import java.util.jar.JarFile;
 import java.util.jar.JarInputStream;
-import java.util.jar.JarOutputStream;
 import java.util.jar.Manifest;
 import java.util.zip.CRC32;
 import java.util.zip.ZipEntry;
 
-import org.springframework.lang.UsesJava7;
+import org.apache.commons.compress.archivers.jar.JarArchiveEntry;
+import org.apache.commons.compress.archivers.jar.JarArchiveOutputStream;
+import org.apache.commons.compress.archivers.zip.UnixStat;
 
 /**
- * Writes JAR content, ensuring valid directory entries are always create and duplicate
+ * Writes JAR content, ensuring valid directory entries are always created and duplicate
  * items are ignored.
  *
  * @author Phillip Webb
  * @author Andy Wilkinson
+ * @since 1.0.0
  */
-public class JarWriter implements LoaderClassesWriter {
+public class JarWriter implements LoaderClassesWriter, AutoCloseable {
+
+	private static final UnpackHandler NEVER_UNPACK = new NeverUnpackHandler();
 
 	private static final String NESTED_LOADER_JAR = "META-INF/loader/spring-boot-loader.jar";
 
 	private static final int BUFFER_SIZE = 32 * 1024;
 
-	private final JarOutputStream jarOutput;
+	private final JarArchiveOutputStream jarOutput;
 
-	private final Set<String> writtenEntries = new HashSet<String>();
+	private final Set<String> writtenEntries = new HashSet<>();
 
 	/**
 	 * Create a new {@link JarWriter} instance.
@@ -78,22 +83,20 @@ public class JarWriter implements LoaderClassesWriter {
 	 * @throws IOException if the file cannot be opened
 	 * @throws FileNotFoundException if the file cannot be found
 	 */
-	public JarWriter(File file, LaunchScript launchScript)
-			throws FileNotFoundException, IOException {
+	public JarWriter(File file, LaunchScript launchScript) throws FileNotFoundException, IOException {
 		FileOutputStream fileOutputStream = new FileOutputStream(file);
 		if (launchScript != null) {
 			fileOutputStream.write(launchScript.toByteArray());
 			setExecutableFilePermission(file);
 		}
-		this.jarOutput = new JarOutputStream(fileOutputStream);
+		this.jarOutput = new JarArchiveOutputStream(fileOutputStream);
+		this.jarOutput.setEncoding("UTF-8");
 	}
 
-	@UsesJava7
 	private void setExecutableFilePermission(File file) {
 		try {
 			Path path = file.toPath();
-			Set<PosixFilePermission> permissions = new HashSet<PosixFilePermission>(
-					Files.getPosixFilePermissions(path));
+			Set<PosixFilePermission> permissions = new HashSet<>(Files.getPosixFilePermissions(path));
 			permissions.add(PosixFilePermission.OWNER_EXECUTE);
 			Files.setPosixFilePermissions(path, permissions);
 		}
@@ -107,14 +110,9 @@ public class JarWriter implements LoaderClassesWriter {
 	 * @param manifest the manifest to write
 	 * @throws IOException of the manifest cannot be written
 	 */
-	public void writeManifest(final Manifest manifest) throws IOException {
-		JarEntry entry = new JarEntry("META-INF/MANIFEST.MF");
-		writeEntry(entry, new EntryWriter() {
-			@Override
-			public void write(OutputStream outputStream) throws IOException {
-				manifest.write(outputStream);
-			}
-		});
+	public void writeManifest(Manifest manifest) throws IOException {
+		JarArchiveEntry entry = new JarArchiveEntry("META-INF/MANIFEST.MF");
+		writeEntry(entry, manifest::write);
 	}
 
 	/**
@@ -123,48 +121,55 @@ public class JarWriter implements LoaderClassesWriter {
 	 * @throws IOException if the entries cannot be written
 	 */
 	public void writeEntries(JarFile jarFile) throws IOException {
-		this.writeEntries(jarFile, new IdentityEntryTransformer());
+		this.writeEntries(jarFile, new IdentityEntryTransformer(), NEVER_UNPACK);
 	}
 
-	void writeEntries(JarFile jarFile, EntryTransformer entryTransformer)
+	void writeEntries(JarFile jarFile, UnpackHandler unpackHandler) throws IOException {
+		this.writeEntries(jarFile, new IdentityEntryTransformer(), unpackHandler);
+	}
+
+	void writeEntries(JarFile jarFile, EntryTransformer entryTransformer, UnpackHandler unpackHandler)
 			throws IOException {
 		Enumeration<JarEntry> entries = jarFile.entries();
 		while (entries.hasMoreElements()) {
-			JarEntry entry = entries.nextElement();
-			ZipHeaderPeekInputStream inputStream = new ZipHeaderPeekInputStream(
-					jarFile.getInputStream(entry));
-			try {
-				if (inputStream.hasZipHeader() && entry.getMethod() != ZipEntry.STORED) {
-					new CrcAndSize(inputStream).setupStoredEntry(entry);
-					inputStream.close();
-					inputStream = new ZipHeaderPeekInputStream(
-							jarFile.getInputStream(entry));
-				}
-				else {
-					entry.setCompressedSize(-1);
-				}
-				EntryWriter entryWriter = new InputStreamEntryWriter(inputStream, true);
-				JarEntry transformedEntry = entryTransformer.transform(entry);
+			JarArchiveEntry entry = new JarArchiveEntry(entries.nextElement());
+			setUpEntry(jarFile, entry);
+			try (ZipHeaderPeekInputStream inputStream = new ZipHeaderPeekInputStream(jarFile.getInputStream(entry))) {
+				EntryWriter entryWriter = new InputStreamEntryWriter(inputStream);
+				JarArchiveEntry transformedEntry = entryTransformer.transform(entry);
 				if (transformedEntry != null) {
-					writeEntry(transformedEntry, entryWriter);
+					writeEntry(transformedEntry, entryWriter, unpackHandler);
 				}
 			}
-			finally {
-				inputStream.close();
+		}
+	}
+
+	private void setUpEntry(JarFile jarFile, JarArchiveEntry entry) throws IOException {
+		try (ZipHeaderPeekInputStream inputStream = new ZipHeaderPeekInputStream(jarFile.getInputStream(entry))) {
+			if (inputStream.hasZipHeader() && entry.getMethod() != ZipEntry.STORED) {
+				new CrcAndSize(inputStream).setupStoredEntry(entry);
+			}
+			else {
+				entry.setCompressedSize(-1);
 			}
 		}
 	}
 
 	/**
 	 * Writes an entry. The {@code inputStream} is closed once the entry has been written
-	 * @param entryName The name of the entry
-	 * @param inputStream The stream from which the entry's data can be read
+	 * @param entryName the name of the entry
+	 * @param inputStream the stream from which the entry's data can be read
 	 * @throws IOException if the write fails
 	 */
 	@Override
 	public void writeEntry(String entryName, InputStream inputStream) throws IOException {
-		JarEntry entry = new JarEntry(entryName);
-		writeEntry(entry, new InputStreamEntryWriter(inputStream, true));
+		JarArchiveEntry entry = new JarArchiveEntry(entryName);
+		try {
+			writeEntry(entry, new InputStreamEntryWriter(inputStream));
+		}
+		finally {
+			inputStream.close();
+		}
 	}
 
 	/**
@@ -173,22 +178,19 @@ public class JarWriter implements LoaderClassesWriter {
 	 * @param library the library
 	 * @throws IOException if the write fails
 	 */
-	public void writeNestedLibrary(String destination, Library library)
-			throws IOException {
+	public void writeNestedLibrary(String destination, Library library) throws IOException {
 		File file = library.getFile();
-		JarEntry entry = new JarEntry(destination + library.getName());
+		JarArchiveEntry entry = new JarArchiveEntry(destination + library.getName());
 		entry.setTime(getNestedLibraryTime(file));
-		if (library.isUnpackRequired()) {
-			entry.setComment("UNPACK:" + FileUtils.sha1Hash(file));
-		}
 		new CrcAndSize(file).setupStoredEntry(entry);
-		writeEntry(entry, new InputStreamEntryWriter(new FileInputStream(file), true));
+		try (FileInputStream input = new FileInputStream(file)) {
+			writeEntry(entry, new InputStreamEntryWriter(input), new LibraryUnpackHandler(library));
+		}
 	}
 
 	private long getNestedLibraryTime(File file) {
 		try {
-			JarFile jarFile = new JarFile(file);
-			try {
+			try (JarFile jarFile = new JarFile(file)) {
 				Enumeration<JarEntry> entries = jarFile.entries();
 				while (entries.hasMoreElements()) {
 					JarEntry entry = entries.nextElement();
@@ -197,9 +199,6 @@ public class JarWriter implements LoaderClassesWriter {
 					}
 				}
 			}
-			finally {
-				jarFile.close();
-			}
 		}
 		catch (Exception ex) {
 			// Ignore and just use the source file timestamp
@@ -225,53 +224,75 @@ public class JarWriter implements LoaderClassesWriter {
 	@Override
 	public void writeLoaderClasses(String loaderJarResourceName) throws IOException {
 		URL loaderJar = getClass().getClassLoader().getResource(loaderJarResourceName);
-		JarInputStream inputStream = new JarInputStream(
-				new BufferedInputStream(loaderJar.openStream()));
-		JarEntry entry;
-		while ((entry = inputStream.getNextJarEntry()) != null) {
-			if (entry.getName().endsWith(".class")) {
-				writeEntry(entry, new InputStreamEntryWriter(inputStream, false));
+		try (JarInputStream inputStream = new JarInputStream(new BufferedInputStream(loaderJar.openStream()))) {
+			JarEntry entry;
+			while ((entry = inputStream.getNextJarEntry()) != null) {
+				if (entry.getName().endsWith(".class")) {
+					writeEntry(new JarArchiveEntry(entry), new InputStreamEntryWriter(inputStream));
+				}
 			}
 		}
-		inputStream.close();
 	}
 
 	/**
 	 * Close the writer.
 	 * @throws IOException if the file cannot be closed
 	 */
+	@Override
 	public void close() throws IOException {
 		this.jarOutput.close();
 	}
 
+	private void writeEntry(JarArchiveEntry entry, EntryWriter entryWriter) throws IOException {
+		writeEntry(entry, entryWriter, NEVER_UNPACK);
+	}
+
 	/**
-	 * Perform the actual write of a {@link JarEntry}. All other {@code write} method
-	 * delegate to this one.
+	 * Perform the actual write of a {@link JarEntry}. All other write methods delegate to
+	 * this one.
 	 * @param entry the entry to write
 	 * @param entryWriter the entry writer or {@code null} if there is no content
+	 * @param unpackHandler handles possible unpacking for the entry
 	 * @throws IOException in case of I/O errors
 	 */
-	private void writeEntry(JarEntry entry, EntryWriter entryWriter) throws IOException {
+	private void writeEntry(JarArchiveEntry entry, EntryWriter entryWriter, UnpackHandler unpackHandler)
+			throws IOException {
 		String parent = entry.getName();
 		if (parent.endsWith("/")) {
 			parent = parent.substring(0, parent.length() - 1);
+			entry.setUnixMode(UnixStat.DIR_FLAG | UnixStat.DEFAULT_DIR_PERM);
 		}
-		if (parent.lastIndexOf("/") != -1) {
-			parent = parent.substring(0, parent.lastIndexOf("/") + 1);
-			if (parent.length() > 0) {
-				writeEntry(new JarEntry(parent), null);
+		else {
+			entry.setUnixMode(UnixStat.FILE_FLAG | UnixStat.DEFAULT_FILE_PERM);
+		}
+		if (parent.lastIndexOf('/') != -1) {
+			parent = parent.substring(0, parent.lastIndexOf('/') + 1);
+			if (!parent.isEmpty()) {
+				writeEntry(new JarArchiveEntry(parent), null, unpackHandler);
 			}
 		}
 
 		if (this.writtenEntries.add(entry.getName())) {
-			this.jarOutput.putNextEntry(entry);
+			entryWriter = addUnpackCommentIfNecessary(entry, entryWriter, unpackHandler);
+			this.jarOutput.putArchiveEntry(entry);
 			if (entryWriter != null) {
 				entryWriter.write(this.jarOutput);
 			}
-			this.jarOutput.closeEntry();
+			this.jarOutput.closeArchiveEntry();
 		}
 	}
 
+	private EntryWriter addUnpackCommentIfNecessary(JarArchiveEntry entry, EntryWriter entryWriter,
+			UnpackHandler unpackHandler) throws IOException {
+		if (entryWriter == null || !unpackHandler.requiresUnpack(entry.getName())) {
+			return entryWriter;
+		}
+		ByteArrayOutputStream output = new ByteArrayOutputStream();
+		entryWriter.write(output);
+		entry.setComment("UNPACK:" + unpackHandler.sha1Hash(entry.getName()));
+		return new InputStreamEntryWriter(new ByteArrayInputStream(output.toByteArray()));
+	}
+
 	/**
 	 * Interface used to write jar entry date.
 	 */
@@ -293,11 +314,8 @@ public class JarWriter implements LoaderClassesWriter {
 
 		private final InputStream inputStream;
 
-		private final boolean close;
-
-		InputStreamEntryWriter(InputStream inputStream, boolean close) {
+		InputStreamEntryWriter(InputStream inputStream) {
 			this.inputStream = inputStream;
-			this.close = close;
 		}
 
 		@Override
@@ -308,9 +326,6 @@ public class JarWriter implements LoaderClassesWriter {
 				outputStream.write(buffer, 0, bytesRead);
 			}
 			outputStream.flush();
-			if (this.close) {
-				this.inputStream.close();
-			}
 		}
 
 	}
@@ -334,13 +349,12 @@ public class JarWriter implements LoaderClassesWriter {
 			super(in);
 			this.header = new byte[4];
 			this.headerLength = in.read(this.header);
-			this.headerStream = new ByteArrayInputStream(this.header, 0,
-					this.headerLength);
+			this.headerStream = new ByteArrayInputStream(this.header, 0, this.headerLength);
 		}
 
 		@Override
 		public int read() throws IOException {
-			int read = (this.headerStream != null ? this.headerStream.read() : -1);
+			int read = (this.headerStream != null) ? this.headerStream.read() : -1;
 			if (read != -1) {
 				this.position++;
 				if (this.position >= this.headerLength) {
@@ -358,8 +372,7 @@ public class JarWriter implements LoaderClassesWriter {
 
 		@Override
 		public int read(byte[] b, int off, int len) throws IOException {
-			int read = (this.headerStream != null ? this.headerStream.read(b, off, len)
-					: -1);
+			int read = (this.headerStream != null) ? this.headerStream.read(b, off, len) : -1;
 			if (read <= 0) {
 				return readRemainder(b, off, len);
 			}
@@ -400,13 +413,9 @@ public class JarWriter implements LoaderClassesWriter {
 		private long size;
 
 		CrcAndSize(File file) throws IOException {
-			FileInputStream inputStream = new FileInputStream(file);
-			try {
+			try (FileInputStream inputStream = new FileInputStream(file)) {
 				load(inputStream);
 			}
-			finally {
-				inputStream.close();
-			}
 		}
 
 		CrcAndSize(InputStream inputStream) throws IOException {
@@ -422,7 +431,7 @@ public class JarWriter implements LoaderClassesWriter {
 			}
 		}
 
-		public void setupStoredEntry(JarEntry entry) {
+		public void setupStoredEntry(JarArchiveEntry entry) {
 			entry.setSize(this.size);
 			entry.setCompressedSize(this.size);
 			entry.setCrc(this.crc.getValue());
@@ -437,7 +446,7 @@ public class JarWriter implements LoaderClassesWriter {
 	 */
 	interface EntryTransformer {
 
-		JarEntry transform(JarEntry jarEntry);
+		JarArchiveEntry transform(JarArchiveEntry jarEntry);
 
 	}
 
@@ -447,10 +456,56 @@ public class JarWriter implements LoaderClassesWriter {
 	private static final class IdentityEntryTransformer implements EntryTransformer {
 
 		@Override
-		public JarEntry transform(JarEntry jarEntry) {
+		public JarArchiveEntry transform(JarArchiveEntry jarEntry) {
 			return jarEntry;
 		}
 
 	}
 
+	/**
+	 * An {@code UnpackHandler} determines whether or not unpacking is required and
+	 * provides a SHA1 hash if required.
+	 */
+	interface UnpackHandler {
+
+		boolean requiresUnpack(String name);
+
+		String sha1Hash(String name) throws IOException;
+
+	}
+
+	private static final class NeverUnpackHandler implements UnpackHandler {
+
+		@Override
+		public boolean requiresUnpack(String name) {
+			return false;
+		}
+
+		@Override
+		public String sha1Hash(String name) {
+			throw new UnsupportedOperationException();
+		}
+
+	}
+
+	private static final class LibraryUnpackHandler implements UnpackHandler {
+
+		private final Library library;
+
+		private LibraryUnpackHandler(Library library) {
+			this.library = library;
+		}
+
+		@Override
+		public boolean requiresUnpack(String name) {
+			return this.library.isUnpackRequired();
+		}
+
+		@Override
+		public String sha1Hash(String name) throws IOException {
+			return FileUtils.sha1Hash(this.library.getFile());
+		}
+
+	}
+
 }
