@@ -5,7 +5,7 @@
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
  *
- *      http://www.apache.org/licenses/LICENSE-2.0
+ *      https://www.apache.org/licenses/LICENSE-2.0
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
@@ -42,7 +42,7 @@ final class StringToNumberConverterFactory implements ConverterFactory<String, N
 
 	@Override
 	public <T extends Number> Converter<String, T> getConverter(Class<T> targetType) {
-		return new StringToNumber<T>(targetType);
+		return new StringToNumber<>(targetType);
 	}
 
 
