@@ -1,11 +1,11 @@
 /*
- * Copyright 2002-2015 the original author or authors.
+ * Copyright 2002-2019 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
  *
- *      http://www.apache.org/licenses/LICENSE-2.0
+ *      https://www.apache.org/licenses/LICENSE-2.0
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
@@ -25,13 +25,14 @@ import org.springframework.expression.Operation;
 import org.springframework.expression.TypedValue;
 import org.springframework.expression.spel.CodeFlow;
 import org.springframework.expression.spel.ExpressionState;
+import org.springframework.util.Assert;
 import org.springframework.util.NumberUtils;
 
 /**
  * Implements the {@code multiply} operator.
  *
  * <p>Conversions and promotions are handled as defined in
- * <a href="http://java.sun.com/docs/books/jls/third_edition/html/conversions.html">Section 5.6.2 of the
+ * <a href="https://java.sun.com/docs/books/jls/third_edition/html/conversions.html">Section 5.6.2 of the
  * Java Language Specification</a>, with the addiction of {@code BigDecimal}/{@code BigInteger} management:
  *
  * <p>If any of the operands is of a reference type, unboxing conversion (Section 5.1.8)
@@ -51,8 +52,8 @@ import org.springframework.util.NumberUtils;
  */
 public class OpMultiply extends Operator {
 
-	public OpMultiply(int pos, SpelNodeImpl... operands) {
-		super("*", pos, operands);
+	public OpMultiply(int startPos, int endPos, SpelNodeImpl... operands) {
+		super("*", startPos, endPos, operands);
 	}
 
 
@@ -125,37 +126,40 @@ public class OpMultiply extends Operator {
 			return false;
 		}
 		if (this.children.length > 1) {
-			 if (!getRightOperand().isCompilable()) {
-				 return false;
-			 }
+			if (!getRightOperand().isCompilable()) {
+				return false;
+			}
 		}
 		return (this.exitTypeDescriptor != null);
 	}
-	
+
 	@Override
 	public void generateCode(MethodVisitor mv, CodeFlow cf) {
 		getLeftOperand().generateCode(mv, cf);
 		String leftDesc = getLeftOperand().exitTypeDescriptor;
-		CodeFlow.insertNumericUnboxOrPrimitiveTypeCoercion(mv, leftDesc, this.exitTypeDescriptor.charAt(0));
+		String exitDesc = this.exitTypeDescriptor;
+		Assert.state(exitDesc != null, "No exit type descriptor");
+		char targetDesc = exitDesc.charAt(0);
+		CodeFlow.insertNumericUnboxOrPrimitiveTypeCoercion(mv, leftDesc, targetDesc);
 		if (this.children.length > 1) {
 			cf.enterCompilationScope();
 			getRightOperand().generateCode(mv, cf);
 			String rightDesc = getRightOperand().exitTypeDescriptor;
 			cf.exitCompilationScope();
-			CodeFlow.insertNumericUnboxOrPrimitiveTypeCoercion(mv, rightDesc, this.exitTypeDescriptor.charAt(0));
-			switch (this.exitTypeDescriptor.charAt(0)) {
+			CodeFlow.insertNumericUnboxOrPrimitiveTypeCoercion(mv, rightDesc, targetDesc);
+			switch (targetDesc) {
 				case 'I':
 					mv.visitInsn(IMUL);
 					break;
 				case 'J':
 					mv.visitInsn(LMUL);
 					break;
-				case 'F': 
+				case 'F':
 					mv.visitInsn(FMUL);
 					break;
 				case 'D':
 					mv.visitInsn(DMUL);
-					break;				
+					break;
 				default:
 					throw new IllegalStateException(
 							"Unrecognized exit type descriptor: '" + this.exitTypeDescriptor + "'");
